//line php7/php7.y:2
package php7

import __yyfmt__ "fmt"

//line php7/php7.y:2
import (
	"strconv"
	"strings"

	"github.com/z7zmey/php-parser/node"
	"github.com/z7zmey/php-parser/node/expr"
	"github.com/z7zmey/php-parser/node/expr/assign"
	"github.com/z7zmey/php-parser/node/expr/binary"
	"github.com/z7zmey/php-parser/node/expr/cast"
	"github.com/z7zmey/php-parser/node/name"
	"github.com/z7zmey/php-parser/node/scalar"
	"github.com/z7zmey/php-parser/node/stmt"
	"github.com/z7zmey/php-parser/scanner"
)

//line php7/php7.y:21
type yySymType struct {
	yys               int
	node              node.Node
	token             scanner.Token
	boolWithToken     boolWithToken
	list              []node.Node
	foreachVariable   foreachVariable
	nodesWithEndToken *nodesWithEndToken
	str               string
	altSyntaxNode     altSyntaxNode
}

const T_INCLUDE = 57346
const T_INCLUDE_ONCE = 57347
const T_EXIT = 57348
const T_IF = 57349
const T_LNUMBER = 57350
const T_DNUMBER = 57351
const T_STRING = 57352
const T_STRING_VARNAME = 57353
const T_VARIABLE = 57354
const T_NUM_STRING = 57355
const T_INLINE_HTML = 57356
const T_CHARACTER = 57357
const T_BAD_CHARACTER = 57358
const T_ENCAPSED_AND_WHITESPACE = 57359
const T_CONSTANT_ENCAPSED_STRING = 57360
const T_ECHO = 57361
const T_DO = 57362
const T_WHILE = 57363
const T_ENDWHILE = 57364
const T_FOR = 57365
const T_ENDFOR = 57366
const T_FOREACH = 57367
const T_ENDFOREACH = 57368
const T_DECLARE = 57369
const T_ENDDECLARE = 57370
const T_AS = 57371
const T_SWITCH = 57372
const T_ENDSWITCH = 57373
const T_CASE = 57374
const T_DEFAULT = 57375
const T_BREAK = 57376
const T_CONTINUE = 57377
const T_GOTO = 57378
const T_FUNCTION = 57379
const T_CONST = 57380
const T_RETURN = 57381
const T_TRY = 57382
const T_CATCH = 57383
const T_FINALLY = 57384
const T_THROW = 57385
const T_USE = 57386
const T_INSTEADOF = 57387
const T_GLOBAL = 57388
const T_VAR = 57389
const T_UNSET = 57390
const T_ISSET = 57391
const T_EMPTY = 57392
const T_HALT_COMPILER = 57393
const T_CLASS = 57394
const T_TRAIT = 57395
const T_INTERFACE = 57396
const T_EXTENDS = 57397
const T_IMPLEMENTS = 57398
const T_OBJECT_OPERATOR = 57399
const T_DOUBLE_ARROW = 57400
const T_LIST = 57401
const T_ARRAY = 57402
const T_CALLABLE = 57403
const T_CLASS_C = 57404
const T_TRAIT_C = 57405
const T_METHOD_C = 57406
const T_FUNC_C = 57407
const T_LINE = 57408
const T_FILE = 57409
const T_COMMENT = 57410
const T_DOC_COMMENT = 57411
const T_OPEN_TAG = 57412
const T_OPEN_TAG_WITH_ECHO = 57413
const T_CLOSE_TAG = 57414
const T_WHITESPACE = 57415
const T_START_HEREDOC = 57416
const T_END_HEREDOC = 57417
const T_DOLLAR_OPEN_CURLY_BRACES = 57418
const T_CURLY_OPEN = 57419
const T_PAAMAYIM_NEKUDOTAYIM = 57420
const T_NAMESPACE = 57421
const T_NS_C = 57422
const T_DIR = 57423
const T_NS_SEPARATOR = 57424
const T_ELLIPSIS = 57425
const T_EVAL = 57426
const T_REQUIRE = 57427
const T_REQUIRE_ONCE = 57428
const T_LOGICAL_OR = 57429
const T_LOGICAL_XOR = 57430
const T_LOGICAL_AND = 57431
const T_INSTANCEOF = 57432
const T_NEW = 57433
const T_CLONE = 57434
const T_ELSEIF = 57435
const T_ELSE = 57436
const T_ENDIF = 57437
const T_PRINT = 57438
const T_YIELD = 57439
const T_STATIC = 57440
const T_ABSTRACT = 57441
const T_FINAL = 57442
const T_PRIVATE = 57443
const T_PROTECTED = 57444
const T_PUBLIC = 57445
const T_INC = 57446
const T_DEC = 57447
const T_YIELD_FROM = 57448
const T_INT_CAST = 57449
const T_DOUBLE_CAST = 57450
const T_STRING_CAST = 57451
const T_ARRAY_CAST = 57452
const T_OBJECT_CAST = 57453
const T_BOOL_CAST = 57454
const T_UNSET_CAST = 57455
const T_COALESCE = 57456
const T_SPACESHIP = 57457
const T_NOELSE = 57458
const T_PLUS_EQUAL = 57459
const T_MINUS_EQUAL = 57460
const T_MUL_EQUAL = 57461
const T_DIV_EQUAL = 57462
const T_CONCAT_EQUAL = 57463
const T_MOD_EQUAL = 57464
const T_AND_EQUAL = 57465
const T_OR_EQUAL = 57466
const T_XOR_EQUAL = 57467
const T_SL_EQUAL = 57468
const T_SR_EQUAL = 57469
const T_POW_EQUAL = 57470
const T_BOOLEAN_OR = 57471
const T_BOOLEAN_AND = 57472
const T_IS_EQUAL = 57473
const T_IS_NOT_EQUAL = 57474
const T_IS_IDENTICAL = 57475
const T_IS_NOT_IDENTICAL = 57476
const T_IS_SMALLER_OR_EQUAL = 57477
const T_IS_GREATER_OR_EQUAL = 57478
const T_SL = 57479
const T_SR = 57480
const T_POW = 57481

var yyToknames = [...]string{
	"$end",
	"error",
	"$unk",
	"T_INCLUDE",
	"T_INCLUDE_ONCE",
	"T_EXIT",
	"T_IF",
	"T_LNUMBER",
	"T_DNUMBER",
	"T_STRING",
	"T_STRING_VARNAME",
	"T_VARIABLE",
	"T_NUM_STRING",
	"T_INLINE_HTML",
	"T_CHARACTER",
	"T_BAD_CHARACTER",
	"T_ENCAPSED_AND_WHITESPACE",
	"T_CONSTANT_ENCAPSED_STRING",
	"T_ECHO",
	"T_DO",
	"T_WHILE",
	"T_ENDWHILE",
	"T_FOR",
	"T_ENDFOR",
	"T_FOREACH",
	"T_ENDFOREACH",
	"T_DECLARE",
	"T_ENDDECLARE",
	"T_AS",
	"T_SWITCH",
	"T_ENDSWITCH",
	"T_CASE",
	"T_DEFAULT",
	"T_BREAK",
	"T_CONTINUE",
	"T_GOTO",
	"T_FUNCTION",
	"T_CONST",
	"T_RETURN",
	"T_TRY",
	"T_CATCH",
	"T_FINALLY",
	"T_THROW",
	"T_USE",
	"T_INSTEADOF",
	"T_GLOBAL",
	"T_VAR",
	"T_UNSET",
	"T_ISSET",
	"T_EMPTY",
	"T_HALT_COMPILER",
	"T_CLASS",
	"T_TRAIT",
	"T_INTERFACE",
	"T_EXTENDS",
	"T_IMPLEMENTS",
	"T_OBJECT_OPERATOR",
	"T_DOUBLE_ARROW",
	"T_LIST",
	"T_ARRAY",
	"T_CALLABLE",
	"T_CLASS_C",
	"T_TRAIT_C",
	"T_METHOD_C",
	"T_FUNC_C",
	"T_LINE",
	"T_FILE",
	"T_COMMENT",
	"T_DOC_COMMENT",
	"T_OPEN_TAG",
	"T_OPEN_TAG_WITH_ECHO",
	"T_CLOSE_TAG",
	"T_WHITESPACE",
	"T_START_HEREDOC",
	"T_END_HEREDOC",
	"T_DOLLAR_OPEN_CURLY_BRACES",
	"T_CURLY_OPEN",
	"T_PAAMAYIM_NEKUDOTAYIM",
	"T_NAMESPACE",
	"T_NS_C",
	"T_DIR",
	"T_NS_SEPARATOR",
	"T_ELLIPSIS",
	"T_EVAL",
	"T_REQUIRE",
	"T_REQUIRE_ONCE",
	"T_LOGICAL_OR",
	"T_LOGICAL_XOR",
	"T_LOGICAL_AND",
	"T_INSTANCEOF",
	"T_NEW",
	"T_CLONE",
	"T_ELSEIF",
	"T_ELSE",
	"T_ENDIF",
	"T_PRINT",
	"T_YIELD",
	"T_STATIC",
	"T_ABSTRACT",
	"T_FINAL",
	"T_PRIVATE",
	"T_PROTECTED",
	"T_PUBLIC",
	"T_INC",
	"T_DEC",
	"T_YIELD_FROM",
	"T_INT_CAST",
	"T_DOUBLE_CAST",
	"T_STRING_CAST",
	"T_ARRAY_CAST",
	"T_OBJECT_CAST",
	"T_BOOL_CAST",
	"T_UNSET_CAST",
	"T_COALESCE",
	"T_SPACESHIP",
	"T_NOELSE",
	"'\"'",
	"'`'",
	"'{'",
	"'}'",
	"';'",
	"':'",
	"'('",
	"')'",
	"'['",
	"']'",
	"'?'",
	"'&'",
	"'-'",
	"'+'",
	"'!'",
	"'~'",
	"'@'",
	"'$'",
	"','",
	"'='",
	"T_PLUS_EQUAL",
	"T_MINUS_EQUAL",
	"T_MUL_EQUAL",
	"T_DIV_EQUAL",
	"T_CONCAT_EQUAL",
	"T_MOD_EQUAL",
	"T_AND_EQUAL",
	"T_OR_EQUAL",
	"T_XOR_EQUAL",
	"T_SL_EQUAL",
	"T_SR_EQUAL",
	"T_POW_EQUAL",
	"T_BOOLEAN_OR",
	"T_BOOLEAN_AND",
	"'|'",
	"'^'",
	"T_IS_EQUAL",
	"T_IS_NOT_EQUAL",
	"T_IS_IDENTICAL",
	"T_IS_NOT_IDENTICAL",
	"'<'",
	"T_IS_SMALLER_OR_EQUAL",
	"'>'",
	"T_IS_GREATER_OR_EQUAL",
	"T_SL",
	"T_SR",
	"'.'",
	"'*'",
	"'/'",
	"'%'",
	"T_POW",
}
var yyStatenames = [...]string{}

const yyEofCode = 1
const yyErrCode = 2
const yyInitialStackSize = 16

//line php7/php7.y:2635
type foreachVariable struct {
	node  node.Node
	byRef bool
}

type nodesWithEndToken struct {
	nodes    []node.Node
	endToken scanner.Token
}

type boolWithToken struct {
	value bool
	token *scanner.Token
}

type altSyntaxNode struct {
	node  node.Node
	isAlt bool
}

//line yacctab:1
var yyExca = [...]int{
	-1, 1,
	1, -1,
	-2, 0,
	-1, 2,
	1, 1,
	-2, 0,
	-1, 44,
	57, 422,
	78, 422,
	119, 422,
	125, 422,
	-2, 417,
	-1, 48,
	123, 425,
	-2, 434,
	-1, 84,
	57, 424,
	78, 424,
	119, 424,
	123, 427,
	125, 424,
	-2, 412,
	-1, 107,
	78, 385,
	-2, 414,
	-1, 229,
	57, 422,
	78, 422,
	119, 422,
	125, 422,
	-2, 313,
	-1, 232,
	123, 427,
	-2, 424,
	-1, 235,
	57, 422,
	78, 422,
	119, 422,
	125, 422,
	-2, 315,
	-1, 354,
	115, 0,
	153, 0,
	154, 0,
	155, 0,
	156, 0,
	-2, 337,
	-1, 355,
	115, 0,
	153, 0,
	154, 0,
	155, 0,
	156, 0,
	-2, 338,
	-1, 356,
	115, 0,
	153, 0,
	154, 0,
	155, 0,
	156, 0,
	-2, 339,
	-1, 357,
	115, 0,
	153, 0,
	154, 0,
	155, 0,
	156, 0,
	-2, 340,
	-1, 358,
	157, 0,
	158, 0,
	159, 0,
	160, 0,
	-2, 341,
	-1, 359,
	157, 0,
	158, 0,
	159, 0,
	160, 0,
	-2, 342,
	-1, 360,
	157, 0,
	158, 0,
	159, 0,
	160, 0,
	-2, 343,
	-1, 361,
	157, 0,
	158, 0,
	159, 0,
	160, 0,
	-2, 344,
	-1, 362,
	115, 0,
	153, 0,
	154, 0,
	155, 0,
	156, 0,
	-2, 345,
	-1, 369,
	124, 163,
	135, 163,
	-2, 422,
	-1, 413,
	124, 462,
	126, 462,
	135, 462,
	-2, 422,
	-1, 417,
	57, 423,
	78, 423,
	119, 423,
	123, 426,
	125, 423,
	-2, 347,
	-1, 431,
	123, 448,
	-2, 415,
	-1, 432,
	123, 450,
	-2, 440,
	-1, 511,
	123, 448,
	-2, 416,
	-1, 512,
	123, 450,
	-2, 441,
	-1, 571,
	124, 213,
	-2, 218,
	-1, 596,
	123, 426,
	-2, 423,
	-1, 648,
	124, 213,
	-2, 218,
	-1, 653,
	124, 183,
	-2, 422,
	-1, 661,
	124, 213,
	-2, 218,
	-1, 686,
	124, 461,
	126, 461,
	135, 461,
	-2, 422,
	-1, 719,
	124, 184,
	-2, 422,
	-1, 739,
	12, 265,
	-2, 268,
	-1, 753,
	93, 208,
	94, 208,
	95, 208,
	-2, 0,
	-1, 778,
	124, 183,
	-2, 422,
	-1, 780,
	124, 186,
	-2, 396,
	-1, 800,
	93, 209,
	94, 209,
	95, 209,
	-2, 0,
	-1, 849,
	31, 199,
	32, 199,
	33, 199,
	120, 199,
	-2, 0,
	-1, 882,
	29, 76,
	-2, 80,
	-1, 885,
	31, 198,
	32, 198,
	33, 198,
	120, 198,
	-2, 0,
	-1, 914,
	124, 213,
	-2, 218,
}

const yyPrivate = 57344

const yyLast = 7331

var yyAct = [...]int{

	28, 130, 825, 838, 662, 861, 376, 578, 741, 436,
	874, 107, 837, 793, 774, 666, 811, 675, 785, 126,
	652, 633, 138, 138, 138, 664, 221, 151, 711, 565,
	700, 313, 667, 322, 5, 186, 532, 632, 368, 573,
	405, 514, 522, 80, 223, 317, 150, 378, 9, 316,
	132, 189, 8, 225, 228, 128, 143, 236, 237, 238,
	239, 240, 147, 2, 241, 242, 243, 244, 245, 246,
	247, 127, 250, 125, 513, 258, 259, 260, 315, 314,
	137, 7, 6, 430, 264, 165, 854, 870, 864, 273,
	274, 892, 276, 277, 842, 684, 841, 269, 230, 230,
	82, 868, 893, 140, 141, 835, 179, 406, 832, 105,
	677, 541, 590, 568, 335, 869, 331, 334, 309, 254,
	105, 111, 833, 677, 308, 302, 329, 759, 336, 307,
	332, 301, 319, 115, 121, 704, 324, 325, 84, 728,
	330, 44, 306, 308, 626, 302, 621, 105, 566, 285,
	287, 190, 556, 411, 337, 338, 339, 340, 341, 342,
	343, 344, 345, 346, 347, 348, 349, 350, 351, 352,
	353, 354, 355, 356, 357, 358, 359, 360, 361, 362,
	295, 364, 366, 165, 370, 321, 266, 372, 730, 780,
	219, 269, 862, 232, 232, 693, 229, 235, 333, 79,
	424, 380, 388, 390, 391, 392, 393, 394, 395, 396,
	397, 398, 399, 400, 401, 689, 407, 402, 138, 404,
	179, 225, 610, 261, 144, 855, 230, 283, 179, 384,
	597, 106, 415, 606, 290, 225, 292, 600, 607, 603,
	601, 299, 106, 585, 305, 410, 926, 409, 888, 820,
	138, 819, 218, 809, 826, 801, 371, 425, 217, 163,
	162, 416, 138, 784, 363, 773, 431, 511, 423, 106,
	230, 727, 523, 524, 717, 698, 525, 179, 696, 688,
	266, 650, 638, 628, 529, 270, 598, 533, 117, 225,
	589, 168, 169, 161, 164, 166, 167, 165, 403, 291,
	914, 230, 164, 166, 167, 165, 284, 828, 863, 782,
	551, 813, 812, 920, 720, 535, 163, 162, 687, 661,
	306, 232, 648, 646, 369, 286, 647, 5, 571, 554,
	418, 552, 414, 385, 383, 561, 288, 151, 669, 670,
	560, 9, 429, 520, 275, 8, 272, 510, 518, 271,
	161, 164, 166, 167, 165, 249, 602, 134, 538, 220,
	112, 117, 216, 105, 550, 232, 564, 184, 413, 519,
	289, 183, 420, 421, 7, 6, 544, 182, 136, 270,
	135, 131, 570, 562, 580, 113, 581, 558, 577, 582,
	583, 575, 751, 387, 579, 559, 232, 188, 420, 427,
	421, 421, 420, 268, 903, 668, 179, 294, 588, 293,
	517, 902, 225, 592, 887, 225, 516, 850, 930, 408,
	929, 821, 543, 815, 546, 572, 157, 156, 179, 609,
	134, 808, 765, 112, 612, 752, 768, 769, 768, 769,
	716, 595, 714, 712, 710, 163, 162, 587, 707, 233,
	555, 540, 181, 178, 537, 738, 591, 386, 740, 120,
	374, 328, 432, 512, 327, 180, 159, 163, 162, 326,
	296, 909, 611, 174, 175, 176, 177, 168, 169, 161,
	164, 166, 167, 165, 844, 106, 807, 153, 154, 158,
	160, 172, 173, 170, 171, 174, 175, 176, 177, 168,
	169, 161, 164, 166, 167, 165, 214, 215, 804, 745,
	746, 747, 744, 743, 742, 802, 761, 138, 616, 738,
	536, 117, 740, 105, 814, 608, 767, 886, 859, 110,
	144, 883, 117, 536, 857, 627, 803, 536, 201, 202,
	203, 204, 206, 207, 208, 209, 210, 211, 212, 213,
	205, 620, 613, 790, 783, 642, 324, 644, 637, 617,
	733, 625, 676, 536, 649, 306, 576, 634, 619, 185,
	5, 109, 419, 745, 746, 747, 744, 743, 742, 298,
	673, 789, 674, 671, 9, 640, 579, 255, 8, 643,
	134, 685, 422, 112, 304, 865, 304, 615, 117, 117,
	658, 105, 196, 197, 114, 651, 114, 289, 691, 233,
	230, 230, 672, 198, 200, 199, 681, 7, 6, 117,
	906, 117, 523, 253, 117, 123, 124, 533, 515, 702,
	191, 117, 263, 304, 231, 262, 234, 230, 718, 574,
	379, 116, 636, 708, 304, 106, 630, 289, 382, 631,
	701, 256, 257, 715, 304, 705, 634, 225, 904, 695,
	671, 722, 703, 699, 697, 46, 726, 546, 134, 546,
	122, 112, 547, 671, 706, 786, 713, 77, 78, 732,
	671, 669, 670, 721, 753, 754, 549, 233, 225, 749,
	134, 545, 528, 112, 748, 255, 300, 542, 230, 146,
	134, 731, 757, 112, 192, 232, 232, 303, 369, 653,
	750, 324, 255, 905, 755, 614, 320, 422, 408, 618,
	690, 225, 701, 106, 634, 894, 762, 280, 281, 794,
	758, 829, 232, 760, 148, 686, 763, 724, 725, 777,
	671, 770, 255, 772, 766, 117, 797, 779, 798, 775,
	579, 791, 546, 800, 787, 796, 795, 546, 546, 256,
	257, 148, 230, 745, 746, 747, 744, 743, 742, 810,
	117, 635, 123, 124, 604, 527, 256, 257, 818, 805,
	129, 816, 768, 769, 119, 824, 146, 548, 827, 536,
	806, 377, 817, 232, 823, 375, 719, 1, 195, 255,
	194, 834, 193, 845, 278, 526, 256, 257, 830, 639,
	187, 255, 38, 849, 739, 736, 422, 771, 768, 769,
	851, 843, 737, 381, 546, 255, 546, 848, 663, 531,
	282, 222, 521, 860, 853, 367, 867, 738, 671, 255,
	740, 846, 858, 871, 252, 873, 880, 866, 839, 884,
	885, 775, 145, 792, 879, 142, 323, 232, 149, 890,
	778, 891, 279, 256, 257, 895, 836, 889, 660, 822,
	251, 898, 840, 645, 896, 256, 257, 880, 899, 928,
	224, 546, 897, 43, 901, 879, 42, 16, 908, 256,
	257, 745, 746, 747, 744, 743, 742, 15, 599, 912,
	267, 49, 48, 256, 257, 913, 108, 50, 83, 81,
	919, 922, 915, 831, 918, 917, 579, 72, 248, 923,
	62, 265, 924, 61, 878, 877, 671, 876, 875, 927,
	735, 4, 931, 88, 89, 70, 47, 93, 94, 36,
	45, 105, 665, 27, 723, 657, 310, 110, 26, 18,
	17, 118, 19, 297, 30, 3, 31, 435, 788, 20,
	729, 0, 0, 21, 22, 35, 37, 13, 23, 33,
	0, 0, 34, 12, 0, 24, 0, 29, 86, 87,
	10, 39, 40, 41, 0, 0, 0, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 0, 0, 0,
	0, 0, 0, 103, 0, 0, 0, 0, 11, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 25, 77, 78,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 104, 73, 14, 629,
	32, 0, 60, 0, 52, 0, 0, 0, 57, 56,
	58, 59, 71, 106, 4, 0, 88, 89, 70, 47,
	93, 94, 36, 0, 105, 0, 27, 0, 0, 0,
	110, 26, 18, 17, 0, 19, 0, 30, 0, 31,
	0, 0, 20, 0, 0, 0, 21, 22, 35, 37,
	13, 23, 33, 0, 0, 34, 12, 0, 24, 0,
	29, 86, 87, 10, 39, 40, 41, 0, 0, 0,
	0, 51, 109, 0, 102, 98, 99, 100, 95, 96,
	0, 0, 0, 0, 0, 0, 103, 0, 0, 0,
	0, 11, 101, 97, 112, 0, 90, 91, 92, 0,
	0, 0, 0, 85, 53, 0, 0, 0, 74, 75,
	25, 77, 78, 0, 0, 0, 54, 55, 76, 63,
	64, 65, 66, 67, 68, 69, 0, 0, 0, 104,
	73, 14, 539, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 4, 0, 88,
	89, 70, 47, 93, 94, 36, 0, 105, 0, 27,
	0, 0, 0, 110, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 13, 23, 33, 0, 0, 34, 12,
	0, 24, 0, 29, 86, 87, 10, 39, 40, 41,
	0, 0, 0, 0, 51, 109, 0, 102, 98, 99,
	100, 95, 96, 0, 0, 0, 0, 0, 0, 103,
	0, 0, 0, 0, 11, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 25, 77, 78, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 104, 73, 14, 0, 32, 0, 60, 0,
	52, 0, 0, 0, 57, 56, 58, 59, 71, 106,
	312, 0, 88, 89, 70, 47, 93, 94, 36, 0,
	105, 0, 27, 0, 0, 0, 110, 26, 18, 17,
	0, 19, 0, 30, 0, 31, 0, 0, 20, 0,
	0, 0, 21, 22, 35, 37, 0, 23, 33, 0,
	0, 34, 0, 0, 24, 0, 29, 86, 87, 318,
	39, 40, 41, 0, 0, 0, 0, 51, 109, 0,
	102, 98, 99, 100, 95, 96, 0, 0, 0, 0,
	0, 0, 103, 0, 0, 0, 0, 134, 101, 97,
	112, 0, 90, 91, 92, 0, 0, 0, 0, 85,
	53, 0, 0, 0, 74, 75, 25, 77, 78, 0,
	0, 0, 54, 55, 76, 63, 64, 65, 66, 67,
	68, 69, 0, 0, 0, 104, 73, 14, 932, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 106, 312, 0, 88, 89, 70, 47, 93,
	94, 36, 0, 105, 0, 27, 0, 0, 0, 110,
	26, 18, 17, 0, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 37, 0,
	23, 33, 0, 0, 34, 0, 0, 24, 0, 29,
	86, 87, 318, 39, 40, 41, 0, 0, 0, 0,
	51, 109, 0, 102, 98, 99, 100, 95, 96, 0,
	0, 0, 0, 0, 0, 103, 0, 0, 0, 0,
	134, 101, 97, 112, 0, 90, 91, 92, 0, 0,
	0, 0, 85, 53, 0, 0, 0, 74, 75, 25,
	77, 78, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 104, 73,
	14, 925, 32, 0, 60, 0, 52, 0, 0, 0,
	57, 56, 58, 59, 71, 106, 312, 0, 88, 89,
	70, 47, 93, 94, 36, 0, 105, 0, 27, 0,
	0, 0, 110, 26, 18, 17, 0, 19, 0, 30,
	0, 31, 0, 0, 20, 0, 0, 0, 21, 22,
	35, 37, 0, 23, 33, 0, 0, 34, 0, 0,
	24, 0, 29, 86, 87, 318, 39, 40, 41, 0,
	0, 0, 0, 51, 109, 0, 102, 98, 99, 100,
	95, 96, 0, 0, 0, 0, 0, 0, 103, 0,
	0, 0, 0, 134, 101, 97, 112, 0, 90, 91,
	92, 0, 0, 0, 0, 85, 53, 0, 0, 0,
	74, 75, 25, 77, 78, 0, 0, 0, 54, 55,
	76, 63, 64, 65, 66, 67, 68, 69, 0, 0,
	0, 104, 73, 14, 921, 32, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 106, 312,
	0, 88, 89, 70, 47, 93, 94, 36, 0, 105,
	0, 27, 0, 0, 0, 110, 26, 18, 17, 0,
	19, 0, 30, 0, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 37, 0, 23, 33, 0, 0,
	34, 0, 0, 24, 0, 29, 86, 87, 318, 39,
	40, 41, 0, 0, 0, 0, 51, 109, 0, 102,
	98, 99, 100, 95, 96, 0, 0, 0, 0, 0,
	0, 103, 0, 0, 0, 0, 134, 101, 97, 112,
	0, 90, 91, 92, 0, 0, 0, 0, 85, 53,
	0, 0, 0, 74, 75, 25, 77, 78, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 104, 73, 14, 911, 32, 0,
	60, 0, 52, 0, 0, 0, 57, 56, 58, 59,
	71, 106, 312, 0, 88, 89, 70, 47, 93, 94,
	36, 0, 105, 0, 27, 0, 0, 0, 110, 26,
	18, 17, 0, 19, 0, 30, 0, 31, 0, 0,
	20, 0, 0, 0, 21, 22, 35, 37, 0, 23,
	33, 0, 0, 34, 0, 0, 24, 0, 29, 86,
	87, 318, 39, 40, 41, 0, 0, 0, 0, 51,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 0, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 25, 77,
	78, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 0, 0, 0, 104, 73, 14,
	910, 32, 0, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 106, 312, 0, 88, 89, 70,
	47, 93, 94, 36, 0, 105, 0, 27, 0, 0,
	0, 110, 26, 18, 17, 0, 19, 907, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	37, 0, 23, 33, 0, 0, 34, 0, 0, 24,
	0, 29, 86, 87, 318, 39, 40, 41, 0, 0,
	0, 0, 51, 109, 0, 102, 98, 99, 100, 95,
	96, 0, 0, 0, 0, 0, 0, 103, 0, 0,
	0, 0, 134, 101, 97, 112, 0, 90, 91, 92,
	0, 0, 0, 0, 85, 53, 0, 0, 0, 74,
	75, 25, 77, 78, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	104, 73, 14, 0, 32, 0, 60, 0, 52, 0,
	0, 0, 57, 56, 58, 59, 71, 106, 312, 0,
	88, 89, 70, 47, 93, 94, 36, 0, 105, 0,
	27, 0, 0, 0, 110, 26, 18, 17, 0, 19,
	0, 30, 0, 31, 0, 0, 20, 0, 0, 0,
	21, 22, 35, 37, 0, 23, 33, 0, 0, 34,
	0, 0, 24, 0, 29, 86, 87, 318, 39, 40,
	41, 0, 0, 0, 0, 51, 109, 0, 102, 98,
	99, 100, 95, 96, 0, 0, 0, 0, 0, 0,
	103, 0, 0, 0, 0, 134, 101, 97, 112, 0,
	90, 91, 92, 0, 0, 0, 0, 85, 53, 0,
	0, 0, 74, 75, 25, 77, 78, 0, 0, 0,
	54, 55, 76, 63, 64, 65, 66, 67, 68, 69,
	0, 0, 0, 104, 73, 14, 856, 32, 0, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	106, 312, 0, 88, 89, 70, 47, 93, 94, 36,
	0, 105, 0, 27, 0, 0, 0, 110, 26, 18,
	17, 0, 19, 0, 30, 852, 31, 0, 0, 20,
	0, 0, 0, 21, 22, 35, 37, 0, 23, 33,
	0, 0, 34, 0, 0, 24, 0, 29, 86, 87,
	318, 39, 40, 41, 0, 0, 0, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 0, 0, 0,
	0, 0, 0, 103, 0, 0, 0, 0, 134, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 25, 77, 78,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 0, 0, 0, 104, 73, 14, 0,
	32, 0, 60, 0, 52, 0, 0, 0, 57, 56,
	58, 59, 71, 106, 312, 0, 88, 89, 70, 47,
	93, 94, 36, 0, 105, 0, 27, 0, 0, 0,
	110, 26, 18, 17, 0, 19, 0, 30, 0, 31,
	781, 0, 20, 0, 0, 0, 21, 22, 35, 37,
	0, 23, 33, 0, 0, 34, 0, 0, 24, 0,
	29, 86, 87, 318, 39, 40, 41, 0, 0, 0,
	0, 51, 109, 0, 102, 98, 99, 100, 95, 96,
	0, 0, 0, 0, 0, 0, 103, 0, 0, 0,
	0, 134, 101, 97, 112, 0, 90, 91, 92, 0,
	0, 0, 0, 85, 53, 0, 0, 0, 74, 75,
	25, 77, 78, 0, 0, 0, 54, 55, 76, 63,
	64, 65, 66, 67, 68, 69, 0, 0, 0, 104,
	73, 14, 0, 32, 0, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 312, 0, 88,
	89, 70, 47, 93, 94, 36, 0, 105, 0, 27,
	0, 0, 0, 110, 26, 18, 17, 764, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 37, 0, 23, 33, 0, 0, 34, 0,
	0, 24, 0, 29, 86, 87, 318, 39, 40, 41,
	0, 0, 0, 0, 51, 109, 0, 102, 98, 99,
	100, 95, 96, 0, 0, 0, 0, 0, 0, 103,
	0, 0, 0, 0, 134, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 25, 77, 78, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 104, 73, 14, 0, 32, 0, 60, 0,
	52, 0, 0, 0, 57, 56, 58, 59, 71, 106,
	312, 0, 88, 89, 70, 47, 93, 94, 36, 0,
	105, 0, 27, 0, 0, 0, 110, 26, 18, 17,
	0, 19, 0, 30, 0, 31, 0, 0, 20, 0,
	0, 0, 21, 22, 35, 37, 0, 23, 33, 0,
	0, 34, 0, 0, 24, 0, 29, 86, 87, 318,
	39, 40, 41, 0, 0, 0, 0, 51, 109, 0,
	102, 98, 99, 100, 95, 96, 0, 0, 0, 0,
	0, 0, 103, 0, 0, 0, 0, 134, 101, 97,
	112, 0, 90, 91, 92, 0, 0, 0, 0, 85,
	53, 0, 0, 680, 74, 75, 25, 77, 78, 0,
	0, 0, 54, 55, 76, 63, 64, 65, 66, 67,
	68, 69, 0, 0, 0, 104, 73, 14, 0, 32,
	0, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 106, 312, 0, 88, 89, 70, 47, 93,
	94, 36, 0, 105, 0, 27, 0, 0, 0, 110,
	26, 18, 17, 0, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 37, 0,
	23, 33, 0, 0, 34, 0, 0, 24, 0, 29,
	86, 87, 318, 39, 40, 41, 0, 0, 0, 0,
	51, 109, 0, 102, 98, 99, 100, 95, 96, 0,
	0, 0, 0, 0, 0, 103, 0, 0, 0, 0,
	134, 101, 97, 112, 0, 90, 91, 92, 0, 0,
	0, 0, 85, 53, 0, 0, 0, 74, 75, 25,
	77, 78, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 104, 73,
	14, 569, 32, 0, 60, 0, 52, 0, 0, 0,
	57, 56, 58, 59, 71, 106, 312, 0, 88, 89,
	70, 47, 93, 94, 36, 0, 105, 0, 27, 0,
	0, 0, 110, 26, 18, 17, 0, 19, 0, 30,
	0, 31, 0, 0, 20, 0, 0, 0, 21, 22,
	35, 37, 0, 23, 33, 0, 0, 34, 0, 0,
	24, 0, 29, 86, 87, 318, 39, 40, 41, 0,
	0, 0, 0, 51, 109, 0, 102, 98, 99, 100,
	95, 96, 0, 0, 0, 0, 0, 0, 103, 0,
	0, 0, 0, 134, 101, 97, 112, 0, 90, 91,
	92, 0, 0, 0, 0, 85, 53, 0, 0, 0,
	74, 75, 25, 77, 78, 0, 0, 0, 54, 55,
	76, 63, 64, 65, 66, 67, 68, 69, 0, 0,
	0, 104, 73, 14, 311, 32, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 106, 312,
	0, 88, 89, 70, 47, 93, 94, 36, 0, 105,
	0, 27, 0, 0, 0, 110, 26, 18, 17, 0,
	19, 0, 30, 0, 31, 0, 0, 20, 0, 0,
	0, 21, 22, 35, 37, 0, 23, 33, 0, 0,
	34, 0, 0, 24, 0, 29, 86, 87, 318, 39,
	40, 41, 0, 0, 0, 0, 51, 109, 0, 102,
	98, 99, 100, 95, 96, 0, 0, 0, 0, 0,
	0, 103, 0, 0, 0, 0, 134, 101, 97, 112,
	0, 90, 91, 92, 0, 0, 0, 0, 85, 53,
	0, 0, 0, 74, 75, 25, 77, 78, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 104, 73, 14, 0, 32, 0,
	60, 0, 52, 0, 0, 0, 57, 56, 58, 59,
	71, 106, 443, 444, 454, 455, 0, 0, 434, 0,
	105, 0, 0, 0, 0, 0, 0, 459, 460, 461,
	462, 463, 464, 465, 466, 467, 468, 469, 489, 490,
	491, 492, 493, 481, 482, 483, 484, 485, 470, 471,
	472, 473, 474, 475, 476, 477, 478, 479, 480, 0,
	501, 499, 500, 496, 497, 0, 0, 488, 494, 495,
	502, 503, 505, 504, 506, 507, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 498, 509, 508,
	0, 0, 445, 446, 447, 448, 449, 450, 451, 452,
	453, 456, 457, 458, 486, 487, 437, 438, 439, 440,
	441, 442, 0, 0, 88, 89, 70, 47, 93, 94,
	36, 0, 105, 0, 27, 0, 0, 433, 110, 26,
	18, 17, 0, 19, 0, 30, 0, 31, 0, 0,
	20, 0, 106, 0, 21, 22, 35, 133, 0, 23,
	33, 0, 0, 34, 0, 0, 24, 0, 29, 86,
	87, 0, 0, 0, 0, 0, 0, 0, 0, 51,
	109, 0, 102, 98, 99, 100, 95, 96, 0, 0,
	0, 0, 0, 0, 103, 0, 0, 0, 0, 134,
	101, 97, 112, 0, 90, 91, 92, 0, 0, 0,
	0, 85, 53, 0, 0, 0, 74, 75, 25, 0,
	0, 0, 0, 0, 54, 55, 76, 63, 64, 65,
	66, 67, 68, 69, 0, 0, 0, 104, 73, 14,
	0, 32, 776, 60, 0, 52, 0, 0, 0, 57,
	56, 58, 59, 71, 106, 88, 89, 70, 47, 93,
	94, 36, 0, 105, 0, 27, 0, 0, 0, 110,
	26, 18, 17, 0, 19, 0, 30, 0, 31, 0,
	0, 20, 0, 0, 0, 21, 22, 35, 133, 0,
	23, 33, 0, 0, 34, 0, 0, 24, 0, 29,
	86, 87, 0, 0, 0, 0, 0, 0, 0, 0,
	51, 109, 0, 102, 98, 99, 100, 95, 96, 0,
	0, 0, 0, 0, 0, 103, 0, 0, 0, 0,
	134, 101, 97, 112, 0, 90, 91, 92, 0, 0,
	0, 0, 85, 53, 0, 0, 0, 74, 75, 25,
	0, 0, 0, 0, 0, 54, 55, 76, 63, 64,
	65, 66, 67, 68, 69, 0, 0, 0, 104, 73,
	14, 0, 32, 847, 60, 0, 52, 0, 0, 0,
	57, 56, 58, 59, 71, 106, 88, 89, 70, 47,
	93, 94, 36, 0, 105, 0, 27, 0, 0, 0,
	110, 26, 18, 17, 0, 19, 0, 30, 0, 31,
	0, 0, 20, 0, 0, 0, 21, 22, 35, 133,
	0, 23, 33, 0, 0, 34, 0, 0, 24, 0,
	29, 86, 87, 0, 0, 0, 0, 0, 0, 0,
	0, 51, 109, 0, 102, 98, 99, 100, 95, 96,
	0, 0, 0, 0, 0, 0, 103, 0, 0, 0,
	0, 134, 101, 97, 112, 0, 90, 91, 92, 0,
	0, 0, 0, 85, 53, 0, 0, 0, 74, 75,
	25, 0, 0, 0, 0, 0, 54, 55, 76, 63,
	64, 65, 66, 67, 68, 69, 0, 0, 0, 104,
	73, 14, 0, 32, 682, 60, 0, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 88, 89, 70,
	47, 93, 94, 36, 0, 105, 0, 27, 0, 0,
	0, 110, 26, 18, 17, 0, 19, 0, 30, 0,
	31, 0, 0, 20, 0, 0, 0, 21, 22, 35,
	133, 0, 23, 33, 0, 0, 34, 0, 0, 24,
	0, 29, 86, 87, 0, 0, 0, 0, 0, 0,
	0, 0, 51, 109, 0, 102, 98, 99, 100, 95,
	96, 0, 0, 0, 0, 0, 0, 103, 0, 0,
	0, 0, 134, 101, 97, 112, 0, 90, 91, 92,
	0, 0, 0, 0, 85, 53, 0, 0, 0, 74,
	75, 25, 0, 0, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	104, 73, 14, 0, 32, 659, 60, 0, 52, 0,
	0, 0, 57, 56, 58, 59, 71, 106, 88, 89,
	70, 47, 93, 94, 36, 0, 105, 0, 27, 0,
	0, 0, 110, 26, 18, 17, 0, 19, 0, 30,
	0, 31, 0, 0, 20, 0, 0, 0, 21, 22,
	35, 133, 0, 23, 33, 0, 0, 34, 0, 0,
	24, 0, 29, 86, 87, 0, 0, 0, 0, 0,
	0, 0, 0, 51, 109, 0, 102, 98, 99, 100,
	95, 96, 0, 0, 0, 0, 0, 0, 103, 0,
	0, 0, 0, 134, 101, 97, 112, 0, 90, 91,
	92, 0, 0, 0, 0, 85, 53, 0, 0, 0,
	74, 75, 25, 0, 0, 0, 0, 0, 54, 55,
	76, 63, 64, 65, 66, 67, 68, 69, 0, 0,
	0, 104, 73, 14, 0, 32, 641, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 106, 88,
	89, 70, 47, 93, 94, 36, 0, 105, 0, 27,
	0, 0, 0, 110, 26, 18, 17, 0, 19, 0,
	30, 0, 31, 0, 0, 20, 0, 0, 0, 21,
	22, 35, 133, 0, 23, 33, 0, 0, 34, 0,
	0, 24, 0, 29, 86, 87, 0, 0, 0, 0,
	0, 0, 0, 0, 51, 109, 0, 102, 98, 99,
	100, 95, 96, 0, 0, 0, 0, 0, 0, 103,
	0, 179, 0, 0, 134, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 25, 0, 0, 178, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 159,
	163, 162, 104, 73, 14, 0, 32, 0, 60, 0,
	52, 0, 0, 0, 57, 56, 58, 59, 71, 106,
	155, 157, 156, 179, 172, 173, 170, 171, 174, 175,
	176, 177, 168, 169, 161, 164, 166, 167, 165, 0,
	0, 0, 0, 0, 0, 0, 0, 181, 178, 738,
	0, 0, 740, 0, 813, 812, 0, 0, 0, 0,
	180, 159, 163, 162, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 155, 157, 156, 179, 0, 0,
	0, 0, 153, 154, 158, 160, 172, 173, 170, 171,
	174, 175, 176, 177, 168, 169, 161, 164, 166, 167,
	165, 181, 178, 745, 746, 747, 744, 743, 742, 0,
	0, 0, 0, 756, 180, 159, 163, 162, 0, 0,
	0, 0, 0, 0, 0, 799, 0, 0, 155, 157,
	156, 179, 0, 0, 0, 0, 153, 154, 158, 160,
	172, 173, 170, 171, 174, 175, 176, 177, 168, 169,
	161, 164, 166, 167, 165, 181, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 709, 0, 0, 180, 159,
	163, 162, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 155, 157, 156, 179, 0, 0, 0, 0,
	153, 154, 158, 160, 172, 173, 170, 171, 174, 175,
	176, 177, 168, 169, 161, 164, 166, 167, 165, 181,
	178, 0, 0, 0, 0, 694, 0, 0, 0, 0,
	0, 0, 180, 159, 163, 162, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 155, 157, 156, 179,
	0, 0, 0, 0, 153, 154, 158, 160, 172, 173,
	170, 171, 174, 175, 176, 177, 168, 169, 161, 164,
	166, 167, 165, 181, 178, 0, 0, 0, 0, 692,
	0, 0, 0, 0, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	155, 157, 156, 179, 0, 0, 0, 0, 153, 154,
	158, 160, 172, 173, 170, 171, 174, 175, 176, 177,
	168, 169, 161, 164, 166, 167, 165, 181, 178, 738,
	0, 0, 740, 683, 0, 0, 0, 0, 0, 0,
	180, 159, 163, 162, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 155, 157, 156, 179, 0, 0,
	0, 0, 153, 154, 158, 160, 172, 173, 170, 171,
	174, 175, 176, 177, 168, 169, 161, 164, 166, 167,
	165, 181, 178, 745, 746, 747, 744, 743, 742, 0,
	0, 679, 0, 0, 180, 159, 163, 162, 0, 0,
	0, 0, 0, 0, 0, 734, 0, 0, 155, 157,
	156, 179, 0, 0, 0, 0, 153, 154, 158, 160,
	172, 173, 170, 171, 174, 175, 176, 177, 168, 169,
	161, 164, 166, 167, 165, 181, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 678, 0, 0, 180, 159,
	163, 162, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 155, 157, 156, 179, 0, 0, 0, 0,
	153, 154, 158, 160, 172, 173, 170, 171, 174, 175,
	176, 177, 168, 169, 161, 164, 166, 167, 165, 181,
	178, 0, 0, 0, 0, 624, 0, 0, 0, 0,
	0, 0, 180, 159, 163, 162, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 155, 157, 156, 179,
	0, 0, 0, 0, 153, 154, 158, 160, 172, 173,
	170, 171, 174, 175, 176, 177, 168, 169, 161, 164,
	166, 167, 165, 181, 178, 0, 0, 0, 0, 0,
	0, 0, 0, 623, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	155, 157, 156, 179, 0, 0, 0, 0, 153, 154,
	158, 160, 172, 173, 170, 171, 174, 175, 176, 177,
	168, 169, 161, 164, 166, 167, 165, 181, 178, 0,
	0, 0, 0, 0, 0, 0, 0, 622, 0, 0,
	180, 159, 163, 162, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 155, 157, 156, 179, 0, 0,
	0, 0, 153, 154, 158, 160, 172, 173, 170, 171,
	174, 175, 176, 177, 168, 169, 161, 164, 166, 167,
	165, 181, 178, 0, 0, 0, 0, 605, 0, 0,
	0, 0, 0, 0, 180, 159, 163, 162, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 155, 157,
	156, 179, 0, 0, 0, 0, 153, 154, 158, 160,
	172, 173, 170, 171, 174, 175, 176, 177, 168, 169,
	161, 164, 166, 167, 165, 181, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 596, 0, 0, 180, 159,
	163, 162, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 155, 157, 156, 179, 0, 0, 0, 0,
	153, 154, 158, 160, 172, 173, 170, 171, 174, 175,
	176, 177, 168, 169, 161, 164, 166, 167, 165, 181,
	178, 0, 0, 0, 0, 586, 0, 0, 0, 0,
	0, 0, 180, 159, 163, 162, 0, 0, 0, 0,
	0, 0, 567, 0, 0, 0, 155, 157, 156, 179,
	0, 0, 0, 0, 153, 154, 158, 160, 172, 173,
	170, 171, 174, 175, 176, 177, 168, 169, 161, 164,
	166, 167, 165, 181, 178, 0, 0, 0, 0, 0,
	0, 0, 0, 584, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	155, 157, 156, 179, 0, 0, 0, 0, 153, 154,
	158, 160, 172, 173, 170, 171, 174, 175, 176, 177,
	168, 169, 161, 164, 166, 167, 165, 181, 178, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	180, 159, 163, 162, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 155, 157, 156, 179, 0, 0,
	0, 0, 153, 154, 158, 160, 172, 173, 170, 171,
	174, 175, 176, 177, 168, 169, 161, 164, 166, 167,
	165, 181, 178, 0, 0, 0, 0, 0, 0, 563,
	0, 0, 0, 0, 180, 159, 163, 162, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 155, 157,
	156, 179, 0, 0, 0, 0, 153, 154, 158, 160,
	172, 173, 170, 171, 174, 175, 176, 177, 168, 169,
	161, 164, 166, 167, 165, 181, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 557, 0, 0, 180, 159,
	163, 162, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 155, 157, 156, 179, 0, 0, 0, 0,
	153, 154, 158, 160, 172, 173, 170, 171, 174, 175,
	176, 177, 168, 169, 161, 164, 166, 167, 165, 181,
	178, 0, 0, 0, 0, 0, 0, 0, 0, 553,
	0, 0, 180, 159, 163, 162, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 155, 157, 156, 179,
	0, 0, 0, 0, 153, 154, 158, 160, 172, 173,
	170, 171, 174, 175, 176, 177, 168, 169, 161, 164,
	166, 167, 165, 181, 178, 412, 0, 0, 0, 0,
	0, 0, 0, 417, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 155, 157, 156, 179, 153, 154,
	158, 160, 172, 173, 170, 171, 174, 175, 176, 177,
	168, 169, 161, 164, 166, 167, 165, 0, 0, 0,
	0, 181, 178, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 180, 159, 163, 162, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 155, 157,
	156, 179, 0, 0, 0, 0, 153, 154, 158, 160,
	172, 173, 170, 171, 174, 175, 176, 177, 168, 169,
	161, 164, 166, 167, 165, 181, 178, 0, 0, 0,
	0, 0, 373, 0, 0, 0, 0, 0, 180, 159,
	163, 162, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 155, 157, 156, 179, 0, 0, 0, 0,
	153, 154, 158, 160, 172, 173, 170, 171, 174, 175,
	176, 177, 168, 169, 161, 164, 166, 167, 165, 181,
	178, 0, 0, 0, 0, 0, 152, 0, 0, 0,
	0, 0, 180, 159, 163, 162, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 155, 157, 156, 179,
	0, 0, 0, 0, 153, 154, 158, 160, 172, 173,
	170, 171, 174, 175, 176, 177, 168, 169, 161, 164,
	166, 167, 165, 181, 178, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 180, 159, 163, 162,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 153, 154,
	158, 160, 172, 173, 170, 171, 174, 175, 176, 177,
	168, 169, 161, 164, 166, 167, 165, 443, 444, 454,
	455, 0, 0, 882, 0, 0, 0, 0, 0, 0,
	0, 0, 459, 460, 461, 462, 463, 464, 465, 466,
	467, 468, 469, 489, 490, 491, 492, 493, 481, 482,
	483, 484, 485, 470, 471, 472, 473, 474, 475, 476,
	477, 478, 479, 480, 0, 501, 499, 500, 496, 497,
	0, 0, 488, 494, 495, 502, 503, 505, 504, 506,
	507, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 881, 509, 508, 112, 0, 445, 446, 447,
	448, 449, 450, 451, 452, 453, 456, 457, 458, 486,
	487, 437, 438, 439, 440, 441, 442, 0, 0, 443,
	444, 454, 455, 0, 0, 882, 0, 0, 0, 0,
	0, 0, 0, 900, 459, 460, 461, 462, 463, 464,
	465, 466, 467, 468, 469, 489, 490, 491, 492, 493,
	481, 482, 483, 484, 485, 470, 471, 472, 473, 474,
	475, 476, 477, 478, 479, 480, 0, 501, 499, 500,
	496, 497, 0, 0, 488, 494, 495, 502, 503, 505,
	504, 506, 507, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 881, 509, 508, 112, 0, 445,
	446, 447, 448, 449, 450, 451, 452, 453, 456, 457,
	458, 486, 487, 437, 438, 439, 440, 441, 442, 156,
	179, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 872, 0, 0, 0, 0,
	0, 0, 428, 0, 181, 178, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 180, 159, 163,
	162, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 179, 0, 0, 0, 0, 153,
	154, 158, 160, 172, 173, 170, 171, 174, 175, 176,
	177, 168, 169, 161, 164, 166, 167, 165, 181, 178,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 180, 159, 163, 162, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 179, 0,
	0, 0, 0, 153, 154, 158, 160, 172, 173, 170,
	171, 174, 175, 176, 177, 168, 169, 161, 164, 166,
	167, 165, 181, 178, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 180, 159, 163, 162, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 179, 0, 0, 0, 0, 153, 154, 158,
	160, 172, 173, 170, 171, 174, 175, 176, 177, 168,
	169, 161, 164, 166, 167, 165, 181, 178, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	159, 163, 162, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 179, 0, 0, 0, 0,
	0, 153, 154, 158, 160, 172, 173, 170, 171, 174,
	175, 176, 177, 168, 169, 161, 164, 166, 167, 165,
	178, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 159, 163, 162, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 179, 0,
	0, 0, 0, 0, 0, 154, 158, 160, 172, 173,
	170, 171, 174, 175, 176, 177, 168, 169, 161, 164,
	166, 167, 165, 178, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 159, 163, 162, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 179, 0, 0, 0, 0, 0, 0, 0, 158,
	160, 172, 173, 170, 171, 174, 175, 176, 177, 168,
	169, 161, 164, 166, 167, 165, 178, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 159,
	163, 162, 0, 0, 0, 0, 88, 89, 70, 0,
	93, 94, 117, 0, 105, 0, 0, 0, 0, 0,
	110, 0, 0, 160, 172, 173, 170, 171, 174, 175,
	176, 177, 168, 169, 161, 164, 166, 167, 165, 133,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 86, 87, 0, 0, 0, 0, 0, 0, 0,
	0, 51, 109, 0, 102, 98, 99, 100, 95, 96,
	0, 0, 0, 0, 0, 0, 103, 0, 0, 0,
	0, 134, 101, 97, 112, 534, 90, 91, 92, 0,
	0, 0, 0, 85, 53, 0, 0, 0, 74, 75,
	139, 0, 0, 0, 0, 0, 54, 55, 76, 63,
	64, 65, 66, 67, 68, 69, 0, 0, 0, 104,
	73, 0, 0, 0, 0, 60, 530, 52, 0, 0,
	0, 57, 56, 58, 59, 71, 106, 88, 89, 70,
	0, 93, 94, 117, 0, 105, 0, 0, 0, 0,
	0, 110, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	133, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 86, 87, 0, 0, 0, 0, 0, 0,
	0, 0, 227, 109, 0, 102, 98, 99, 100, 95,
	96, 0, 0, 0, 0, 0, 0, 103, 0, 0,
	0, 0, 134, 101, 97, 112, 0, 90, 91, 92,
	0, 0, 0, 0, 85, 53, 0, 0, 0, 74,
	75, 139, 0, 0, 0, 0, 0, 54, 55, 76,
	63, 64, 65, 66, 67, 68, 69, 0, 0, 0,
	104, 73, 0, 0, 0, 0, 60, 0, 52, 0,
	0, 226, 57, 56, 58, 59, 71, 106, 88, 89,
	70, 0, 93, 94, 117, 0, 105, 0, 0, 0,
	0, 0, 110, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 133, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 86, 87, 0, 0, 0, 0, 0,
	0, 0, 0, 51, 109, 0, 102, 98, 99, 100,
	95, 96, 0, 0, 0, 0, 0, 0, 103, 0,
	0, 0, 0, 134, 101, 97, 112, 534, 90, 91,
	92, 0, 0, 0, 0, 85, 53, 0, 0, 0,
	74, 75, 139, 0, 0, 0, 0, 0, 54, 55,
	76, 63, 64, 65, 66, 67, 68, 69, 0, 0,
	0, 104, 73, 0, 0, 0, 0, 60, 0, 52,
	0, 0, 0, 57, 56, 58, 59, 71, 106, 88,
	89, 70, 0, 93, 94, 117, 0, 105, 0, 0,
	0, 0, 0, 110, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 133, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 86, 87, 0, 0, 0, 0,
	0, 0, 0, 0, 594, 109, 0, 102, 98, 99,
	100, 95, 96, 0, 0, 0, 0, 0, 0, 103,
	0, 0, 0, 0, 134, 101, 97, 112, 0, 90,
	91, 92, 0, 0, 0, 0, 85, 53, 0, 0,
	0, 74, 75, 139, 0, 0, 0, 0, 0, 54,
	55, 76, 63, 64, 65, 66, 67, 68, 69, 0,
	0, 0, 104, 73, 0, 0, 0, 0, 60, 0,
	52, 0, 0, 593, 57, 56, 58, 59, 71, 106,
	88, 89, 70, 0, 93, 94, 117, 426, 105, 0,
	0, 0, 0, 0, 110, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 133, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 86, 87, 0, 0, 0,
	0, 0, 0, 0, 0, 51, 109, 0, 102, 98,
	99, 100, 95, 96, 0, 0, 0, 0, 0, 0,
	103, 0, 0, 0, 0, 134, 101, 97, 112, 0,
	90, 91, 92, 0, 0, 0, 0, 85, 53, 0,
	0, 0, 74, 75, 139, 0, 0, 0, 0, 0,
	54, 55, 76, 63, 64, 65, 66, 67, 68, 69,
	0, 0, 0, 104, 73, 0, 0, 0, 0, 60,
	0, 52, 0, 0, 0, 57, 56, 58, 59, 71,
	106, 88, 89, 70, 0, 93, 94, 117, 0, 105,
	0, 0, 0, 0, 0, 110, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 133, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 86, 87, 0, 0,
	0, 0, 0, 0, 0, 0, 51, 109, 0, 102,
	98, 99, 100, 95, 96, 0, 0, 0, 0, 0,
	0, 103, 0, 0, 0, 0, 134, 101, 97, 112,
	0, 90, 91, 92, 0, 0, 0, 0, 85, 53,
	0, 0, 0, 74, 75, 139, 0, 0, 0, 0,
	0, 54, 55, 76, 63, 64, 65, 66, 67, 68,
	69, 0, 0, 0, 104, 73, 0, 0, 0, 0,
	60, 0, 52, 0, 0, 389, 57, 56, 58, 59,
	71, 106, 88, 89, 70, 0, 93, 94, 117, 0,
	105, 0, 0, 0, 0, 0, 110, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 133, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 86, 87, 0,
	0, 0, 0, 0, 0, 0, 0, 51, 109, 0,
	102, 98, 99, 100, 95, 96, 0, 0, 0, 0,
	0, 0, 103, 0, 0, 0, 0, 134, 101, 97,
	112, 0, 90, 91, 92, 0, 0, 0, 0, 85,
	53, 0, 0, 0, 74, 75, 139, 0, 0, 0,
	0, 0, 54, 55, 76, 63, 64, 65, 66, 67,
	68, 69, 0, 0, 0, 104, 73, 0, 0, 0,
	365, 60, 0, 52, 0, 0, 0, 57, 56, 58,
	59, 71, 106, 88, 89, 70, 0, 93, 94, 117,
	0, 105, 0, 0, 0, 0, 0, 110, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 133, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 86, 87,
	0, 0, 0, 0, 0, 0, 0, 0, 51, 109,
	0, 102, 98, 99, 100, 95, 96, 0, 0, 0,
	0, 0, 0, 103, 179, 0, 0, 0, 134, 101,
	97, 112, 0, 90, 91, 92, 0, 0, 0, 0,
	85, 53, 0, 0, 0, 74, 75, 139, 0, 178,
	0, 0, 0, 54, 55, 76, 63, 64, 65, 66,
	67, 68, 69, 163, 162, 0, 104, 73, 0, 0,
	0, 0, 60, 0, 52, 0, 0, 0, 57, 56,
	58, 59, 71, 106, 0, 0, 0, 172, 173, 170,
	171, 174, 175, 176, 177, 168, 169, 161, 164, 166,
	167, 165, 443, 444, 454, 455, 0, 0, 434, 0,
	0, 0, 0, 0, 0, 0, 0, 459, 460, 461,
	462, 463, 464, 465, 466, 467, 468, 469, 489, 490,
	491, 492, 493, 481, 482, 483, 484, 485, 470, 471,
	472, 473, 474, 475, 476, 477, 478, 479, 480, 0,
	501, 499, 500, 496, 497, 0, 0, 488, 494, 495,
	502, 503, 505, 504, 506, 507, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 498, 509, 508,
	0, 0, 445, 446, 447, 448, 449, 450, 451, 452,
	453, 456, 457, 458, 486, 487, 437, 438, 439, 440,
	441, 442, 443, 444, 454, 455, 0, 0, 916, 0,
	0, 0, 0, 0, 0, 0, 0, 459, 460, 461,
	462, 463, 464, 465, 466, 467, 468, 469, 489, 490,
	491, 492, 493, 481, 482, 483, 484, 485, 470, 471,
	472, 473, 474, 475, 476, 477, 478, 479, 480, 0,
	501, 499, 500, 496, 497, 0, 0, 488, 494, 495,
	502, 503, 505, 504, 506, 507, 117, 0, 105, 0,
	0, 0, 0, 0, 110, 0, 0, 498, 509, 508,
	0, 0, 445, 446, 447, 448, 449, 450, 451, 452,
	453, 456, 457, 458, 486, 487, 745, 746, 747, 744,
	743, 742, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 655, 109, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 134, 0, 0, 112, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 233, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 231,
	0, 656, 0, 0, 654, 0, 0, 0, 0, 0,
	106,
}
var yyPact = [...]int{

	-1000, -1000, 1195, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	262, 522, 588, 770, -1000, -1000, -1000, 258, 3945, 257,
	255, 6889, 6889, 6889, 135, 749, 6889, -1000, 5185, 254,
	248, 244, -1000, 450, 6889, 800, 275, 23, 578, 792,
	790, 788, 509, 520, 402, -1000, -1000, 239, -1000, -1000,
	133, 236, 6103, 6889, 511, 511, 6889, 6889, 6889, 6889,
	6889, -1000, -1000, 6889, 6889, 6889, 6889, 6889, 6889, 6889,
	232, 6889, -1000, 827, 6889, 6889, 6889, -1000, -1000, -1000,
	98, -1000, 557, 554, -1000, 351, 226, 223, 6889, 6889,
	221, 6889, 6889, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, 787, 813, -1000, 108, 202, 202, 213,
	-1000, 525, 760, 175, 760, 288, -1000, -1000, 349, 614,
	10, 625, 760, -1000, -1000, -1000, -1000, 8, -1000, -18,
	2924, 6889, 695, 23, 524, 6889, 6889, 348, 5239, 662,
	343, 340, 5, -1000, -1000, -5, 23, -1000, -19, -7,
	-1000, 5239, -1000, 6889, 6889, 6889, 6889, 6889, 6889, 6889,
	6889, 6889, 6889, 6889, 6889, 6889, 6889, 6889, 6889, 6889,
	6889, 6889, 6889, 6889, 6889, 6889, 6889, 6889, 6889, 589,
	6758, 6889, 511, 6889, 770, -1000, 5131, 339, -1000, 785,
	-1000, 781, -1000, 585, -1000, 593, 211, 3945, 210, 336,
	271, 6627, 6889, 6889, 6889, 6889, 6889, 6889, 6889, 6889,
	6889, 6889, 6889, 6889, -1000, -1000, 6889, 6889, 6889, 97,
	6103, 119, 18, -1000, -1000, 5077, 511, 209, -1000, -1000,
	98, 6889, -1000, -1000, 6103, -1000, -82, -82, 16, -82,
	5019, -82, -82, -82, -82, -82, -82, -82, -1000, 6889,
	-82, 454, 683, 799, -1000, 143, 6496, 511, 5628, 5574,
	5628, 6889, 3188, 3188, 202, -1000, 550, 291, 202, -1000,
	-1000, 6889, 6889, 5239, 5239, 6889, 5239, 5239, 730, -1000,
	700, 575, 683, 6889, -1000, -1000, 5972, -1000, 6103, 779,
	525, 333, 525, -1000, -1000, 1062, -1000, 330, -10, 615,
	760, -1000, 609, 553, 777, 604, -1000, -1000, 770, 6889,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, 208, 4965,
	206, -1000, 329, 17, 5239, 4911, -1000, -1000, -1000, -1000,
	135, -1000, 722, -1000, 6889, -1000, 6889, 5735, 5788, 338,
	5628, 5520, 5841, 6874, 3931, 138, 138, 138, 16, -82,
	16, 16, 187, 187, 316, 316, 316, 316, 130, 130,
	130, 130, 316, -1000, 4857, 6889, 5682, 13, -1000, -1000,
	4803, -11, 2791, -1000, -1000, -1000, 205, 585, 583, 611,
	447, -1000, 611, 6889, -1000, 6889, -1000, -1000, 5628, 6889,
	5628, 5628, 5628, 5628, 5628, 5628, 5628, 5628, 5628, 5628,
	5628, 5628, 4749, 117, 4695, 202, -1000, 6889, -1000, 166,
	-24, 6103, 6365, -1000, 6103, 4641, 104, -1000, 162, -1000,
	-1000, -1000, -1000, 227, 764, 4587, 113, 405, 6889, 96,
	202, -1000, -1000, 6889, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	202, -1000, -1000, -1000, -1000, 135, 6889, 6889, 97, 135,
	585, 11, -1000, 5239, 4533, 4479, -1000, -1000, -1000, 4425,
	-1000, 9, -1000, 5239, 6889, 159, -1000, -1000, 929, -1000,
	-1000, -1000, 527, 567, -1000, 760, 565, 735, -1000, 523,
	-1000, 5239, 158, 3814, 6889, 6889, 6889, 204, -1000, -1000,
	199, 5239, -1000, 6889, 5682, 157, 511, 7196, 3683, -1000,
	196, 278, 583, -1000, 611, -1000, -1000, 443, -12, -1000,
	4371, 4317, 2658, 6874, 3552, -1000, -1000, -1000, 4263, -41,
	6889, -1000, 5239, 511, 195, 155, -1000, -1000, -1000, 89,
	-1000, -1000, 707, -1000, -1000, -1000, -1000, 6889, -1000, 5628,
	-1000, -1000, 4209, -1000, -1000, 69, 4155, -1000, -1000, 583,
	154, 6889, -1000, -1000, -1000, 151, 6234, 5239, -1000, -1000,
	760, 510, 0, -1000, -1000, 760, 735, -1000, 327, -1000,
	-1000, -1000, 4101, 323, 5239, -1000, 322, 321, 278, 5682,
	319, -1000, 150, 580, 511, 191, 6103, -1000, -1000, -1000,
	696, 278, 147, 4, -1000, 60, -1000, -1000, 621, -1000,
	-1000, -1000, -1000, 441, -12, 4335, -1000, 611, 3945, 270,
	314, -1000, -1000, -1000, 6889, 5628, -1000, 6103, -41, -1000,
	-1000, 4047, -1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000,
	-8, -1000, 760, 396, 735, -1000, 0, -1000, 2525, 311,
	6889, 406, -1000, 786, -1000, 141, -1000, 3290, 7196, -1000,
	6103, 63, 2392, -1000, 186, 435, 139, 631, 278, 498,
	-1000, -1000, 434, -1000, -1000, -1000, 717, 718, 611, 665,
	-1000, -1000, -1000, -1000, -1000, -1000, -1000, -1000, 4065, -1000,
	-1000, -1000, -1000, 3057, 5628, 131, 395, 417, 388, 760,
	-8, -1000, -1000, 366, 310, -1000, 129, -1000, 6889, 190,
	404, 302, 750, 631, -1000, -1000, -1000, 127, -1000, 125,
	-1000, 300, 611, -1000, 132, 132, 184, -1000, 719, -1000,
	-1000, 793, -13, -1000, -31, 7038, 23, -25, -1000, -1000,
	3057, -41, -1000, -1000, -1000, -1000, 364, -1000, -1000, 3421,
	3993, -1000, -1000, -1000, -1000, -1000, 296, 132, 2259, 3290,
	-1000, -1000, 74, -1000, 2126, 415, 278, 409, 180, -48,
	475, -1000, -1000, 717, -1000, 6889, -20, -1000, -49, 7038,
	-1000, -1000, 5505, 411, -1000, -1000, -1000, -1000, -1000, 3057,
	-1000, 408, 293, -1000, 124, 611, -1000, -1000, -1000, -1000,
	-33, -1000, -1000, 713, 6889, -1000, -1000, 5239, -1000, 7038,
	6889, -1000, -1000, 5403, -1000, 290, 283, 613, 684, 542,
	-1000, 524, -1000, -1000, 1993, 3057, -1000, -1000, 352, -1000,
	1860, 1727, -1000, 180, -1000, 5239, -1000, -1000, 5239, 177,
	-1000, -1000, -1000, -1000, 611, 7138, 7038, 192, 1594, -1000,
	-1000, -1000, -1000, -1000, 278, -12, -1000, -1000, 7038, -1000,
	-1000, -1000, 1461, 122, -1000, -1000, 132, 299, -1000, -1000,
	-1000, 1328, -1000,
}
var yyPgo = [...]int{

	0, 960, 958, 51, 9, 957, 3, 955, 11, 31,
	79, 78, 49, 45, 953, 21, 951, 73, 19, 55,
	946, 0, 80, 945, 944, 38, 141, 47, 25, 942,
	36, 940, 56, 62, 930, 10, 928, 927, 925, 924,
	13, 46, 923, 921, 100, 84, 199, 920, 918, 917,
	5, 909, 83, 40, 908, 138, 43, 907, 906, 902,
	901, 900, 119, 898, 897, 887, 886, 12, 883, 880,
	44, 42, 32, 2, 15, 665, 8, 771, 20, 879,
	873, 872, 41, 74, 623, 870, 121, 869, 868, 866,
	71, 858, 33, 856, 855, 30, 37, 853, 852, 28,
	845, 835, 459, 833, 18, 832, 831, 26, 829, 63,
	1, 4, 828, 17, 823, 39, 822, 815, 814, 7,
	812, 6, 809, 803, 14, 797, 29, 16,
}
var yyR1 = [...]int{

	0, 125, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 4,
	4, 4, 4, 4, 4, 4, 4, 4, 4, 5,
	5, 5, 5, 5, 5, 5, 6, 6, 109, 109,
	86, 86, 8, 8, 8, 7, 7, 7, 7, 7,
	7, 7, 7, 7, 7, 7, 7, 7, 7, 7,
	77, 77, 14, 14, 16, 16, 126, 126, 96, 96,
	95, 95, 102, 102, 15, 15, 18, 18, 17, 17,
	90, 90, 110, 110, 20, 20, 20, 20, 20, 20,
	20, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 9, 9, 9, 9, 9,
	9, 9, 9, 9, 9, 88, 88, 87, 87, 24,
	24, 101, 101, 25, 10, 1, 1, 2, 2, 11,
	11, 120, 120, 75, 75, 12, 13, 27, 27, 114,
	114, 115, 115, 78, 78, 78, 78, 123, 123, 124,
	124, 23, 23, 80, 80, 80, 80, 99, 99, 99,
	127, 127, 122, 122, 66, 66, 64, 64, 68, 68,
	65, 65, 111, 111, 112, 112, 28, 28, 29, 29,
	74, 74, 72, 72, 72, 73, 73, 82, 82, 108,
	108, 30, 30, 94, 94, 32, 98, 98, 33, 33,
	113, 113, 34, 34, 34, 34, 119, 119, 81, 81,
	81, 100, 100, 35, 35, 36, 37, 37, 37, 37,
	39, 39, 38, 79, 79, 117, 117, 116, 116, 118,
	118, 76, 76, 76, 76, 76, 76, 97, 97, 40,
	40, 89, 89, 67, 19, 91, 91, 41, 92, 92,
	93, 93, 43, 42, 42, 31, 31, 31, 31, 31,
	31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
	31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
	31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
	31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
	31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
	31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
	31, 31, 31, 31, 31, 31, 31, 31, 31, 31,
	31, 121, 3, 3, 104, 104, 103, 103, 50, 50,
	51, 51, 51, 51, 44, 44, 45, 45, 48, 48,
	85, 85, 85, 83, 83, 55, 55, 55, 49, 49,
	49, 49, 49, 49, 49, 49, 49, 49, 49, 49,
	49, 49, 49, 49, 56, 56, 56, 21, 21, 22,
	22, 54, 57, 57, 57, 58, 58, 58, 59, 59,
	59, 59, 59, 59, 26, 26, 26, 46, 46, 46,
	60, 60, 61, 61, 61, 61, 61, 61, 52, 52,
	52, 53, 53, 53, 107, 70, 70, 106, 106, 69,
	69, 69, 69, 69, 69, 84, 84, 84, 84, 62,
	62, 62, 62, 62, 62, 62, 63, 63, 63, 63,
	47, 47, 47, 47, 47, 47, 47, 105, 105, 71,
}
var yyR2 = [...]int{

	0, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 2, 0,
	1, 3, 1, 3, 2, 1, 1, 1, 1, 1,
	1, 4, 3, 5, 4, 3, 4, 3, 4, 3,
	1, 1, 6, 7, 6, 7, 0, 1, 3, 1,
	3, 1, 3, 1, 1, 2, 1, 3, 1, 2,
	3, 1, 2, 0, 1, 1, 1, 1, 1, 1,
	4, 3, 1, 1, 5, 7, 9, 5, 3, 3,
	3, 3, 3, 3, 1, 2, 6, 7, 9, 5,
	1, 6, 3, 3, 2, 0, 9, 1, 3, 0,
	4, 1, 3, 1, 11, 0, 1, 0, 1, 9,
	8, 1, 2, 1, 1, 6, 7, 0, 2, 0,
	2, 0, 2, 1, 2, 4, 3, 1, 4, 1,
	4, 1, 4, 3, 4, 4, 5, 0, 5, 4,
	1, 1, 1, 4, 5, 6, 1, 3, 6, 7,
	3, 6, 1, 0, 1, 3, 4, 6, 0, 1,
	1, 2, 1, 1, 1, 0, 2, 2, 4, 1,
	3, 1, 2, 3, 1, 1, 3, 1, 1, 3,
	2, 0, 3, 4, 3, 10, 1, 3, 1, 2,
	3, 1, 2, 2, 2, 3, 3, 3, 4, 3,
	1, 1, 3, 1, 3, 1, 1, 0, 1, 1,
	2, 1, 1, 1, 1, 1, 1, 3, 1, 2,
	4, 3, 1, 4, 4, 3, 1, 1, 0, 1,
	3, 1, 8, 3, 2, 6, 5, 3, 4, 2,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 2, 2, 2, 2, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 2, 2, 2, 2, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 1, 5,
	4, 3, 1, 2, 2, 2, 2, 2, 2, 2,
	2, 2, 1, 3, 2, 1, 2, 4, 2, 11,
	12, 0, 0, 1, 0, 4, 3, 1, 1, 2,
	2, 4, 4, 2, 1, 1, 1, 1, 0, 3,
	0, 1, 1, 0, 1, 4, 3, 1, 1, 1,
	1, 1, 1, 1, 1, 1, 1, 1, 3, 2,
	3, 3, 1, 1, 1, 3, 3, 1, 1, 0,
	1, 1, 1, 3, 1, 1, 3, 1, 1, 4,
	4, 4, 4, 1, 1, 1, 3, 1, 4, 2,
	3, 3, 1, 4, 4, 3, 3, 3, 1, 3,
	1, 1, 3, 1, 1, 0, 1, 3, 1, 3,
	1, 4, 2, 6, 4, 2, 2, 1, 2, 1,
	4, 3, 3, 3, 6, 3, 1, 1, 2, 1,
	5, 4, 2, 2, 4, 2, 2, 1, 3, 1,
}
var yyChk = [...]int{

	-1000, -125, -109, -7, 2, -9, -10, -11, -12, -13,
	51, 79, 44, 38, 119, -64, -65, 21, 20, 23,
	30, 34, 35, 39, 46, 98, 19, 14, -21, 48,
	25, 27, 121, 40, 43, 36, 10, 37, -120, 52,
	53, 54, -66, -68, -26, -31, -75, 7, -59, -60,
	-57, 59, 125, 92, 104, 105, 130, 129, 131, 132,
	123, -42, -47, 107, 108, 109, 110, 111, 112, 113,
	6, 133, -49, 118, 96, 97, 106, 99, 100, -46,
	-56, -51, -44, -54, -55, 91, 49, 50, 4, 5,
	84, 85, 86, 8, 9, 66, 67, 81, 63, 64,
	65, 80, 62, 74, 117, 12, 134, -8, -58, 60,
	18, -86, 82, 123, 82, -86, 119, 10, -16, -77,
	-102, -86, 82, 37, 38, -17, -18, -90, -19, 10,
	-110, 123, -9, 37, 79, 123, 123, -22, -21, 98,
	-22, -22, -94, -32, -46, -98, 37, -33, 12, -91,
	-41, -21, 121, 149, 150, 87, 89, 88, 151, 128,
	152, 163, 130, 129, 164, 167, 165, 166, 161, 162,
	155, 156, 153, 154, 157, 158, 159, 160, 115, 90,
	127, 114, 123, 123, 123, 119, -21, 10, 122, -3,
	128, 52, -75, 10, 10, 10, 93, 94, 93, 95,
	94, 136, 137, 138, 139, 148, 140, 141, 142, 143,
	144, 145, 146, 147, 104, 105, 123, 125, 119, 57,
	123, -107, -106, -70, -69, -21, 128, 59, -21, -26,
	-56, 123, -55, 98, 125, -26, -21, -21, -21, -21,
	-21, -21, -21, -21, -21, -21, -21, -21, -48, 123,
	-21, -85, 17, -84, -62, 12, 76, 77, -21, -21,
	-21, 125, 78, 78, -45, -43, -44, -61, 52, -8,
	-46, 123, 123, -21, -21, 123, -21, -21, 17, 75,
	-84, -84, 17, 119, -46, -82, 123, -82, 123, 82,
	-86, 124, -86, 121, 119, -109, 121, -14, -102, -86,
	82, 121, 135, 82, 29, -86, -18, 121, 135, 136,
	-20, 120, 2, -9, -10, -11, -12, -13, 51, -21,
	21, -3, -92, -93, -21, -21, 121, 121, 121, 121,
	135, 121, 135, -3, 136, 121, 135, -21, -21, -21,
	-21, -21, -21, -21, -21, -21, -21, -21, -21, -21,
	-21, -21, -21, -21, -21, -21, -21, -21, -21, -21,
	-21, -21, -21, -45, -21, 122, -21, -101, -25, -26,
	-21, -90, -110, 121, 121, 10, -121, 10, -27, 55,
	-121, -114, 55, 123, -9, 123, 121, 122, -21, 128,
	-21, -21, -21, -21, -21, -21, -21, -21, -21, -21,
	-21, -21, -21, -22, -21, -53, 10, 119, -46, -107,
	126, 135, 58, -26, 123, -21, -107, 124, -22, 118,
	-62, -62, 17, 125, 57, -21, 11, -26, 58, -22,
	-52, -6, -46, 119, 10, -5, -4, 98, 99, 100,
	101, 102, 103, 4, 5, 84, 85, 86, 87, 88,
	89, 90, 91, 92, 6, 7, 93, 94, 95, 19,
	20, 21, 22, 23, 24, 25, 26, 27, 28, 29,
	40, 41, 42, 43, 44, 45, 46, 47, 48, 49,
	50, 35, 36, 37, 38, 39, 96, 97, 59, 30,
	31, 32, 33, 34, 60, 61, 55, 56, 79, 53,
	54, 52, 62, 63, 65, 64, 66, 67, 81, 80,
	-52, -6, -46, -83, -82, 78, 125, 119, 57, 78,
	-83, -105, -71, -21, -21, -21, 75, 75, 117, -21,
	124, -108, -30, -21, 83, -107, 10, 121, -109, 120,
	121, 121, 82, -86, -17, 82, -86, 119, 10, 82,
	-19, -21, 123, 124, 123, 121, 135, 124, -32, -33,
	-121, -21, -41, 122, -21, -126, 135, 29, 124, 120,
	-121, 123, -27, -115, 56, -8, 119, -121, -119, -8,
	-21, -21, -110, -21, 124, 126, 120, -82, -21, 124,
	136, -70, -21, 128, 59, -107, 124, 126, 124, -63,
	10, 13, 129, 12, 10, 120, 120, 125, 120, -21,
	126, -82, -21, -82, -46, -22, -21, -53, -46, -27,
	-126, 135, 124, 124, 120, -126, 135, -21, 124, 120,
	119, 82, -96, -15, -18, -77, 119, -121, 124, -122,
	-9, 122, -21, -92, -21, -80, 119, 122, 123, -21,
	124, -25, -78, -26, 128, 59, 125, -23, -9, 122,
	-88, 123, -111, -112, -28, -29, -74, -72, 127, 60,
	61, -8, -115, -121, -119, -113, 119, 135, 124, 124,
	95, -9, 122, 120, 136, -21, -26, 123, 124, 126,
	13, -21, 120, 126, 120, -115, 124, -71, 124, -30,
	-95, -18, 119, -126, 135, -18, -96, 121, -110, 124,
	121, -99, 121, -99, 121, -111, 121, 124, 58, -26,
	123, -107, -110, -24, 41, 42, -111, 124, 135, -1,
	128, -72, -121, 119, 120, -34, -117, -116, 44, -118,
	47, -76, 103, 102, 101, 98, 99, 100, -113, -8,
	-9, 122, 121, -110, -21, -107, 126, -121, -126, 135,
	-95, 120, -15, -126, 22, 121, -92, 120, 32, 33,
	-99, 31, -99, 124, -124, -9, 122, -78, -26, -107,
	126, 28, 123, 119, 124, -104, 44, -28, -2, 83,
	119, -113, -97, -40, 12, 38, 37, -119, -76, 120,
	-110, 124, 120, 119, 120, -18, -126, 120, 121, 124,
	-21, -127, 122, 121, 120, 121, 31, -104, -110, 124,
	124, 121, -87, -8, -110, -73, 122, -73, 123, 12,
	-113, 120, 121, 135, -121, 136, -89, -67, -6, -3,
	-81, 121, 119, -113, 120, -123, -9, 122, -127, -110,
	121, -73, 26, -124, 12, 151, 120, 119, -74, 119,
	-103, -50, 12, 128, 136, 120, -40, -21, 121, 135,
	136, -6, 120, -100, -35, -36, -37, -38, -39, -8,
	-6, 79, 10, 120, -110, -110, 119, 121, 124, -8,
	-110, -110, 124, 135, 12, -21, -121, -67, -21, -121,
	120, -35, 121, 121, 45, 29, 78, 24, -110, 119,
	120, 120, -50, -121, 123, -119, 10, -4, -76, -6,
	121, 120, -110, -111, -6, 120, 124, -73, -79, 121,
	119, -110, 120,
}
var yyDef = [...]int{

	79, -2, -2, 78, 85, 86, 87, 88, 89, 90,
	0, 0, 0, 0, 123, 132, 133, 0, 0, 0,
	0, 419, 419, 419, 0, 384, 0, 144, 0, 0,
	0, 0, 150, 0, 0, 0, 80, 372, 0, 0,
	0, 0, 206, 0, -2, 418, 171, 0, -2, 435,
	421, 0, 455, 0, 0, 0, 0, 0, 0, 0,
	0, 348, 352, 0, 0, 0, 0, 0, 0, 0,
	388, 0, 362, 390, 0, 365, 0, 173, 174, 428,
	413, 433, 0, 0, -2, 0, 0, 0, 0, 0,
	0, 0, 0, 398, 399, 400, 401, 402, 403, 404,
	405, 406, 407, 0, 0, 437, 0, -2, 0, 0,
	397, 82, 0, 0, 0, 0, 79, 80, 0, 0,
	0, 116, 0, 100, 101, 113, 118, 0, 121, 0,
	0, 0, 0, 372, 0, 288, 0, 0, 420, 384,
	0, 0, 0, 234, 235, 0, 372, 237, 238, 0,
	286, 287, 145, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 123, 0, 0, 154, 371,
	373, 0, 172, 177, 371, 179, 0, 0, 0, 0,
	0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	0, 0, 0, 0, 312, 314, 0, 419, 0, 0,
	455, 0, 454, 458, 456, 460, 0, 0, 299, -2,
	0, 0, -2, 384, 455, -2, 333, 334, 335, 336,
	0, 353, 354, 355, 356, 357, 358, 359, 360, 419,
	361, 0, 391, 392, 467, 469, 0, 0, 364, 366,
	368, 419, 0, 0, 393, 294, 386, 387, 393, 385,
	442, 0, 0, 482, 483, 0, 485, 486, 0, 409,
	0, 0, 0, 0, 439, 380, 0, 383, 455, 0,
	84, 0, 83, 92, 79, 0, 95, 0, 0, 116,
	0, 97, 0, 0, 0, 116, 119, 99, 0, 0,
	122, 131, 124, 125, 126, 127, 128, 129, 0, 0,
	0, 371, 0, 289, 291, 0, 138, 139, 140, 141,
	0, 142, 0, 371, 0, 143, 0, 316, 317, 318,
	319, 320, 321, 322, 323, 324, 325, 326, 327, 328,
	329, 330, 331, 332, -2, -2, -2, -2, -2, -2,
	-2, -2, -2, 346, 0, 0, 351, 106, 161, -2,
	0, 0, 0, 152, 153, 371, 0, 177, 181, 0,
	0, 371, 0, 0, 207, 0, 210, 123, 297, 0,
	300, 301, 302, 303, 304, 305, 306, 307, 308, 309,
	310, 311, 0, 0, 0, 436, 451, 0, 453, 0,
	396, 455, 0, -2, 455, 0, 0, -2, 0, 363,
	468, 465, 466, 0, 0, 0, 0, 422, 0, 0,
	0, -2, -2, 0, 76, 77, 69, 70, 71, 72,
	73, 74, 75, 2, 3, 4, 5, 6, 7, 8,
	9, 10, 11, 12, 13, 14, 15, 16, 17, 18,
	19, 20, 21, 22, 23, 24, 25, 26, 27, 28,
	29, 30, 31, 32, 33, 34, 35, 36, 37, 38,
	39, 40, 41, 42, 43, 44, 45, 46, 47, 48,
	49, 50, 51, 52, 53, 54, 55, 56, 57, 58,
	59, 60, 61, 62, 63, 64, 65, 66, 67, 68,
	0, -2, -2, 293, 394, 0, 419, 0, 0, 0,
	177, 106, 487, 489, 0, 0, 408, 411, 410, 0,
	227, 106, 229, 231, 0, 0, 81, 91, 0, 94,
	96, 98, 0, 116, 112, 0, 116, 0, 117, 0,
	120, 371, 0, 0, 0, 288, 0, 0, 233, 236,
	0, 239, 285, 0, 350, 0, 107, 0, 0, 155,
	0, -2, 181, 371, 0, 178, 241, 0, 180, 246,
	0, 0, 0, 298, 0, 429, 431, 432, 0, 0,
	0, 457, 459, 0, 0, 0, -2, 396, 389, 0,
	476, 477, 0, 479, 471, 472, 473, 0, 475, 367,
	430, 381, 0, 382, 446, 0, 0, 445, 447, 181,
	0, 107, 481, 484, 438, 0, 107, 232, 395, 93,
	0, 0, 106, 109, 114, 0, 0, 284, 0, 134,
	202, 123, 0, 0, 290, 137, 197, 197, -2, 349,
	0, 162, 0, -2, 0, 0, 455, 149, 191, 123,
	159, -2, 0, 212, 214, 165, 219, 220, 0, 222,
	223, 224, 371, 0, 182, 267, 241, 0, 0, 0,
	0, 204, 123, 452, 0, 296, -2, 455, 464, 470,
	478, 0, 449, 443, 444, 371, 480, 488, 228, 230,
	106, 111, 0, 0, 107, 115, 106, 130, 0, 0,
	288, 0, 197, 0, 197, 0, 146, 0, 0, -2,
	455, 0, 0, 151, 0, 0, 0, 374, 218, 167,
	166, 221, 0, 241, 175, 240, 0, 0, 0, -2,
	266, 269, 271, 272, 273, 274, 275, 276, 267, 247,
	205, 123, 211, -2, 295, 0, 0, 0, 0, 107,
	106, 104, 108, 0, 0, 135, 0, 193, 0, 0,
	0, 0, 0, 374, 147, 189, 123, 0, -2, 0,
	-2, 0, 0, 123, 225, 225, 0, 215, 0, 168,
	241, 267, 0, 278, 371, 0, 372, 0, 270, 176,
	-2, 463, 474, 241, 102, 110, 0, 105, 203, 0,
	0, 123, 200, 201, 194, 195, 0, 225, 0, 0,
	185, 192, 0, 157, 0, 0, 0, 0, 0, 216,
	267, 170, 242, 0, 279, 0, 0, 282, 0, 0,
	244, 248, 0, 267, 103, 136, 187, 123, 123, -2,
	196, 0, 0, 148, 0, 0, 160, 123, 226, 123,
	0, 377, 378, 0, 0, 169, 277, 371, 243, 0,
	0, 371, 249, 0, 251, 0, 0, 261, 0, 0,
	260, 57, -2, 292, 0, -2, 123, 190, 0, 158,
	0, 0, 375, 0, 379, 217, 280, 281, 371, 0,
	250, 252, 253, 254, 0, 0, 0, 0, 0, 123,
	164, 369, 376, 283, -2, 255, 256, 257, 259, 262,
	188, 370, 0, 0, 258, 156, 225, 0, 245, 263,
	123, 0, 264,
}
var yyTok1 = [...]int{

	1, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 131, 117, 3, 134, 166, 128, 3,
	123, 124, 164, 130, 135, 129, 163, 165, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 122, 121,
	157, 136, 159, 127, 133, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 125, 3, 126, 152, 3, 118, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 3, 3, 3, 3, 3, 3, 3,
	3, 3, 3, 119, 151, 120, 132,
}
var yyTok2 = [...]int{

	2, 3, 4, 5, 6, 7, 8, 9, 10, 11,
	12, 13, 14, 15, 16, 17, 18, 19, 20, 21,
	22, 23, 24, 25, 26, 27, 28, 29, 30, 31,
	32, 33, 34, 35, 36, 37, 38, 39, 40, 41,
	42, 43, 44, 45, 46, 47, 48, 49, 50, 51,
	52, 53, 54, 55, 56, 57, 58, 59, 60, 61,
	62, 63, 64, 65, 66, 67, 68, 69, 70, 71,
	72, 73, 74, 75, 76, 77, 78, 79, 80, 81,
	82, 83, 84, 85, 86, 87, 88, 89, 90, 91,
	92, 93, 94, 95, 96, 97, 98, 99, 100, 101,
	102, 103, 104, 105, 106, 107, 108, 109, 110, 111,
	112, 113, 114, 115, 116, 137, 138, 139, 140, 141,
	142, 143, 144, 145, 146, 147, 148, 149, 150, 153,
	154, 155, 156, 158, 160, 161, 162, 167,
}
var yyTok3 = [...]int{
	0,
}

var yyErrorMessages = [...]struct {
	state int
	token int
	msg   string
}{}

//line yaccpar:1

/*	parser for yacc output	*/

var (
	yyDebug        = 0
	yyErrorVerbose = false
)

type yyLexer interface {
	Lex(lval *yySymType) int
	Error(s string)
}

type yyParser interface {
	Parse(yyLexer) int
	Lookahead() int
}

type yyParserImpl struct {
	lval  yySymType
	stack [yyInitialStackSize]yySymType
	char  int
}

func (p *yyParserImpl) Lookahead() int {
	return p.char
}

func yyNewParser() yyParser {
	return &yyParserImpl{}
}

const yyFlag = -1000

func yyTokname(c int) string {
	if c >= 1 && c-1 < len(yyToknames) {
		if yyToknames[c-1] != "" {
			return yyToknames[c-1]
		}
	}
	return __yyfmt__.Sprintf("tok-%v", c)
}

func yyStatname(s int) string {
	if s >= 0 && s < len(yyStatenames) {
		if yyStatenames[s] != "" {
			return yyStatenames[s]
		}
	}
	return __yyfmt__.Sprintf("state-%v", s)
}

func yyErrorMessage(state, lookAhead int) string {
	const TOKSTART = 4

	if !yyErrorVerbose {
		return "syntax error"
	}

	for _, e := range yyErrorMessages {
		if e.state == state && e.token == lookAhead {
			return "syntax error: " + e.msg
		}
	}

	res := "syntax error: unexpected " + yyTokname(lookAhead)

	// To match Bison, suggest at most four expected tokens.
	expected := make([]int, 0, 4)

	// Look for shiftable tokens.
	base := yyPact[state]
	for tok := TOKSTART; tok-1 < len(yyToknames); tok++ {
		if n := base + tok; n >= 0 && n < yyLast && yyChk[yyAct[n]] == tok {
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}
	}

	if yyDef[state] == -2 {
		i := 0
		for yyExca[i] != -1 || yyExca[i+1] != state {
			i += 2
		}

		// Look for tokens that we accept or reduce.
		for i += 2; yyExca[i] >= 0; i += 2 {
			tok := yyExca[i]
			if tok < TOKSTART || yyExca[i+1] == 0 {
				continue
			}
			if len(expected) == cap(expected) {
				return res
			}
			expected = append(expected, tok)
		}

		// If the default action is to accept or reduce, give up.
		if yyExca[i+1] != 0 {
			return res
		}
	}

	for i, tok := range expected {
		if i == 0 {
			res += ", expecting "
		} else {
			res += " or "
		}
		res += yyTokname(tok)
	}
	return res
}

func yylex1(lex yyLexer, lval *yySymType) (char, token int) {
	token = 0
	char = lex.Lex(lval)
	if char <= 0 {
		token = yyTok1[0]
		goto out
	}
	if char < len(yyTok1) {
		token = yyTok1[char]
		goto out
	}
	if char >= yyPrivate {
		if char < yyPrivate+len(yyTok2) {
			token = yyTok2[char-yyPrivate]
			goto out
		}
	}
	for i := 0; i < len(yyTok3); i += 2 {
		token = yyTok3[i+0]
		if token == char {
			token = yyTok3[i+1]
			goto out
		}
	}

out:
	if token == 0 {
		token = yyTok2[1] /* unknown char */
	}
	if yyDebug >= 3 {
		__yyfmt__.Printf("lex %s(%d)\n", yyTokname(token), uint(char))
	}
	return char, token
}

func yyParse(yylex yyLexer) int {
	return yyNewParser().Parse(yylex)
}

func (yyrcvr *yyParserImpl) Parse(yylex yyLexer) int {
	var yyn int
	var yyVAL yySymType
	var yyDollar []yySymType
	_ = yyDollar // silence set and not used
	yyS := yyrcvr.stack[:]

	Nerrs := 0   /* number of errors */
	Errflag := 0 /* error recovery flag */
	yystate := 0
	yyrcvr.char = -1
	yytoken := -1 // yyrcvr.char translated into internal numbering
	defer func() {
		// Make sure we report no lookahead when not parsing.
		yystate = -1
		yyrcvr.char = -1
		yytoken = -1
	}()
	yyp := -1
	goto yystack

ret0:
	return 0

ret1:
	return 1

yystack:
	/* put a state and value onto the stack */
	if yyDebug >= 4 {
		__yyfmt__.Printf("char %v in %v\n", yyTokname(yytoken), yyStatname(yystate))
	}

	yyp++
	if yyp >= len(yyS) {
		nyys := make([]yySymType, len(yyS)*2)
		copy(nyys, yyS)
		yyS = nyys
	}
	yyS[yyp] = yyVAL
	yyS[yyp].yys = yystate

yynewstate:
	yyn = yyPact[yystate]
	if yyn <= yyFlag {
		goto yydefault /* simple state */
	}
	if yyrcvr.char < 0 {
		yyrcvr.char, yytoken = yylex1(yylex, &yyrcvr.lval)
	}
	yyn += yytoken
	if yyn < 0 || yyn >= yyLast {
		goto yydefault
	}
	yyn = yyAct[yyn]
	if yyChk[yyn] == yytoken { /* valid shift */
		yyrcvr.char = -1
		yytoken = -1
		yyVAL = yyrcvr.lval
		yystate = yyn
		if Errflag > 0 {
			Errflag--
		}
		goto yystack
	}

yydefault:
	/* default state action */
	yyn = yyDef[yystate]
	if yyn == -2 {
		if yyrcvr.char < 0 {
			yyrcvr.char, yytoken = yylex1(yylex, &yyrcvr.lval)
		}

		/* look through exception table */
		xi := 0
		for {
			if yyExca[xi+0] == -1 && yyExca[xi+1] == yystate {
				break
			}
			xi += 2
		}
		for xi += 2; ; xi += 2 {
			yyn = yyExca[xi+0]
			if yyn < 0 || yyn == yytoken {
				break
			}
		}
		yyn = yyExca[xi+1]
		if yyn < 0 {
			goto ret0
		}
	}
	if yyn == 0 {
		/* error ... attempt to resume parsing */
		switch Errflag {
		case 0: /* brand new error */
			yylex.Error(yyErrorMessage(yystate, yytoken))
			Nerrs++
			if yyDebug >= 1 {
				__yyfmt__.Printf("%s", yyStatname(yystate))
				__yyfmt__.Printf(" saw %s\n", yyTokname(yytoken))
			}
			fallthrough

		case 1, 2: /* incompletely recovered error ... try again */
			Errflag = 3

			/* find a state where "error" is a legal shift action */
			for yyp >= 0 {
				yyn = yyPact[yyS[yyp].yys] + yyErrCode
				if yyn >= 0 && yyn < yyLast {
					yystate = yyAct[yyn] /* simulate a shift of "error" */
					if yyChk[yystate] == yyErrCode {
						goto yystack
					}
				}

				/* the current p has no shift on "error", pop stack */
				if yyDebug >= 2 {
					__yyfmt__.Printf("error recovery pops state %d\n", yyS[yyp].yys)
				}
				yyp--
			}
			/* there is no state on the stack with an error shift ... abort */
			goto ret1

		case 3: /* no shift yet; clobber input char */
			if yyDebug >= 2 {
				__yyfmt__.Printf("error recovery discards %s\n", yyTokname(yytoken))
			}
			if yytoken == yyEofCode {
				goto ret1
			}
			yyrcvr.char = -1
			yytoken = -1
			goto yynewstate /* try again in the same state */
		}
	}

	/* reduction by production yyn */
	if yyDebug >= 2 {
		__yyfmt__.Printf("reduce %v in:\n\t%v\n", yyn, yyStatname(yystate))
	}

	yynt := yyn
	yypt := yyp
	_ = yypt // guard against "declared and not used"

	yyp -= yyR2[yyn]
	// yyp is now the index of $0. Perform the default action. Iff the
	// reduced production is ε, $1 is possibly out of range.
	if yyp+1 >= len(yyS) {
		nyys := make([]yySymType, len(yyS)*2)
		copy(nyys, yyS)
		yyS = nyys
	}
	yyVAL = yyS[yyp+1]

	/* consult goto table to find next state */
	yyn = yyR1[yyn]
	yyg := yyPgo[yyn]
	yyj := yyg + yyS[yyp].yys + 1

	if yyj >= yyLast {
		yystate = yyAct[yyg]
	} else {
		yystate = yyAct[yyj]
		if yyChk[yystate] != -yyn {
			yystate = yyAct[yyg]
		}
	}
	// dummy call; replaced with literal code
	switch yynt {

	case 1:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:257
		{
			yylex.(*Parser).rootNode = stmt.NewStmtList(yyDollar[1].list)
			yylex.(*Parser).positions.AddPosition(yylex.(*Parser).rootNode, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
		}
	case 2:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:264
		{
			yyVAL.token = yyDollar[1].token
		}
	case 3:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:264
		{
			yyVAL.token = yyDollar[1].token
		}
	case 4:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:264
		{
			yyVAL.token = yyDollar[1].token
		}
	case 5:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:264
		{
			yyVAL.token = yyDollar[1].token
		}
	case 6:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:264
		{
			yyVAL.token = yyDollar[1].token
		}
	case 7:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:264
		{
			yyVAL.token = yyDollar[1].token
		}
	case 8:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:264
		{
			yyVAL.token = yyDollar[1].token
		}
	case 9:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:264
		{
			yyVAL.token = yyDollar[1].token
		}
	case 10:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 11:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 12:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 13:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 14:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 15:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 16:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 17:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 18:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 19:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 20:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 21:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:265
		{
			yyVAL.token = yyDollar[1].token
		}
	case 22:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 23:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 24:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 25:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 26:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 27:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 28:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 29:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 30:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 31:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:266
		{
			yyVAL.token = yyDollar[1].token
		}
	case 32:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 33:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 34:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 35:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 36:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 37:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 38:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 39:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 40:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 41:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:267
		{
			yyVAL.token = yyDollar[1].token
		}
	case 42:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 43:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 44:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 45:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 46:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 47:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 48:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 49:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 50:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 51:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 52:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:268
		{
			yyVAL.token = yyDollar[1].token
		}
	case 53:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:269
		{
			yyVAL.token = yyDollar[1].token
		}
	case 54:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:269
		{
			yyVAL.token = yyDollar[1].token
		}
	case 55:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:269
		{
			yyVAL.token = yyDollar[1].token
		}
	case 56:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:269
		{
			yyVAL.token = yyDollar[1].token
		}
	case 57:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:269
		{
			yyVAL.token = yyDollar[1].token
		}
	case 58:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:269
		{
			yyVAL.token = yyDollar[1].token
		}
	case 59:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:269
		{
			yyVAL.token = yyDollar[1].token
		}
	case 60:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:269
		{
			yyVAL.token = yyDollar[1].token
		}
	case 61:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:270
		{
			yyVAL.token = yyDollar[1].token
		}
	case 62:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:270
		{
			yyVAL.token = yyDollar[1].token
		}
	case 63:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:270
		{
			yyVAL.token = yyDollar[1].token
		}
	case 64:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:270
		{
			yyVAL.token = yyDollar[1].token
		}
	case 65:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:270
		{
			yyVAL.token = yyDollar[1].token
		}
	case 66:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:270
		{
			yyVAL.token = yyDollar[1].token
		}
	case 67:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:270
		{
			yyVAL.token = yyDollar[1].token
		}
	case 68:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:270
		{
			yyVAL.token = yyDollar[1].token
		}
	case 69:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:274
		{
			yyVAL.token = yyDollar[1].token
		}
	case 70:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:275
		{
			yyVAL.token = yyDollar[1].token
		}
	case 71:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:275
		{
			yyVAL.token = yyDollar[1].token
		}
	case 72:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:275
		{
			yyVAL.token = yyDollar[1].token
		}
	case 73:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:275
		{
			yyVAL.token = yyDollar[1].token
		}
	case 74:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:275
		{
			yyVAL.token = yyDollar[1].token
		}
	case 75:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:275
		{
			yyVAL.token = yyDollar[1].token
		}
	case 76:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:279
		{
			yyVAL.token = yyDollar[1].token
		}
	case 77:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:280
		{
			yyVAL.token = yyDollar[1].token
		}
	case 78:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:285
		{
			if yyDollar[2].node != nil {
				yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
			}
		}
	case 79:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:290
		{
			yyVAL.list = []node.Node{}
		}
	case 80:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:295
		{
			namePart := name.NewNamePart(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(namePart, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.list = []node.Node{namePart}
			yylex.(*Parser).comments.AddComments(namePart, yyDollar[1].token.Comments())
		}
	case 81:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:302
		{
			namePart := name.NewNamePart(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(namePart, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.list = append(yyDollar[1].list, namePart)
			yylex.(*Parser).comments.AddComments(namePart, yyDollar[3].token.Comments())
		}
	case 82:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:312
		{
			yyVAL.node = name.NewName(yyDollar[1].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 83:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:318
		{
			yyVAL.node = name.NewRelative(yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[3].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 84:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:324
		{
			yyVAL.node = name.NewFullyQualified(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[2].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 85:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:333
		{
			// error
			yyVAL.node = nil
		}
	case 86:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:337
		{
			yyVAL.node = yyDollar[1].node
		}
	case 87:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:338
		{
			yyVAL.node = yyDollar[1].node
		}
	case 88:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:339
		{
			yyVAL.node = yyDollar[1].node
		}
	case 89:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:340
		{
			yyVAL.node = yyDollar[1].node
		}
	case 90:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:341
		{
			yyVAL.node = yyDollar[1].node
		}
	case 91:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:342
		{
			yyVAL.node = stmt.NewHaltCompiler()
		}
	case 92:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:344
		{
			name := name.NewName(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node = stmt.NewNamespace(name, nil)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(name, yylex.(*Parser).listGetFirstNodeComments(yyDollar[2].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 93:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:354
		{
			name := name.NewName(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node = stmt.NewNamespace(name, yyDollar[4].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].token))

			yylex.(*Parser).comments.AddComments(name, yylex.(*Parser).listGetFirstNodeComments(yyDollar[2].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 94:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:364
		{
			yyVAL.node = stmt.NewNamespace(nil, yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 95:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:369
		{
			yyVAL.node = yyDollar[2].node
		}
	case 96:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:370
		{
			yyVAL.node = yyDollar[3].node.(*stmt.GroupUse).SetUseType(yyDollar[2].node)
		}
	case 97:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:372
		{
			yyVAL.node = stmt.NewUseList(nil, yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 98:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:377
		{
			yyVAL.node = stmt.NewUseList(yyDollar[2].node, yyDollar[3].list)
		}
	case 99:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:379
		{
			yyVAL.node = stmt.NewConstList(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 100:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:388
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 101:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:394
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 102:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:403
		{
			name := name.NewName(yyDollar[1].list)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[4].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[6].token))

			yylex.(*Parser).comments.AddComments(name, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 103:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:413
		{
			name := name.NewName(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[5].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			yylex.(*Parser).comments.AddComments(name, yylex.(*Parser).listGetFirstNodeComments(yyDollar[2].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 104:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:426
		{
			name := name.NewName(yyDollar[1].list)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[4].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[6].token))

			yylex.(*Parser).comments.AddComments(name, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 105:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:436
		{
			name := name.NewName(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[2].list))
			yyVAL.node = stmt.NewGroupUse(nil, name, yyDollar[5].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			yylex.(*Parser).comments.AddComments(name, yylex.(*Parser).listGetFirstNodeComments(yyDollar[2].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 108:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:454
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 109:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:455
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 110:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:460
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 111:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:461
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 112:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:465
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 113:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:466
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 114:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:470
		{
			yyVAL.node = yyDollar[1].node
		}
	case 115:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:471
		{
			yyVAL.node = yyDollar[2].node.(*stmt.Use).SetUseType(yyDollar[1].node)
		}
	case 116:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:476
		{
			name := name.NewName(yyDollar[1].list)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			yyVAL.node = stmt.NewUse(nil, name, nil)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))

			yylex.(*Parser).comments.AddComments(name, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 117:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:486
		{
			name := name.NewName(yyDollar[1].list)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[1].list))
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(alias, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = stmt.NewUse(nil, name, alias)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(name, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
			yylex.(*Parser).comments.AddComments(alias, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 118:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:501
		{
			yyVAL.node = yyDollar[1].node
		}
	case 119:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:502
		{
			yyVAL.node = yyDollar[2].node
		}
	case 120:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:506
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 121:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:507
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 122:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:512
		{
			if yyDollar[2].node != nil {
				yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
			}
		}
	case 123:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:517
		{
			yyVAL.list = []node.Node{}
		}
	case 124:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:522
		{
			// error
			yyVAL.node = nil
		}
	case 125:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:526
		{
			yyVAL.node = yyDollar[1].node
		}
	case 126:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:527
		{
			yyVAL.node = yyDollar[1].node
		}
	case 127:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:528
		{
			yyVAL.node = yyDollar[1].node
		}
	case 128:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:529
		{
			yyVAL.node = yyDollar[1].node
		}
	case 129:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:530
		{
			yyVAL.node = yyDollar[1].node
		}
	case 130:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:532
		{
			yyVAL.node = stmt.NewHaltCompiler()
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 131:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:540
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 132:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:545
		{
			yyVAL.node = yyDollar[1].node
		}
	case 133:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:546
		{
			yyVAL.node = yyDollar[1].node
		}
	case 134:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:548
		{
			if yyDollar[5].altSyntaxNode.isAlt {
				yyVAL.node = stmt.NewAltWhile(yyDollar[3].node, yyDollar[5].altSyntaxNode.node)
			} else {
				yyVAL.node = stmt.NewWhile(yyDollar[3].node, yyDollar[5].altSyntaxNode.node)
			}
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].altSyntaxNode.node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 135:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:558
		{
			yyVAL.node = stmt.NewDo(yyDollar[2].node, yyDollar[5].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 136:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line php7/php7.y:564
		{
			if yyDollar[9].altSyntaxNode.isAlt {
				yyVAL.node = stmt.NewAltFor(yyDollar[3].list, yyDollar[5].list, yyDollar[7].list, yyDollar[9].altSyntaxNode.node)
			} else {
				yyVAL.node = stmt.NewFor(yyDollar[3].list, yyDollar[5].list, yyDollar[7].list, yyDollar[9].altSyntaxNode.node)
			}
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[9].altSyntaxNode.node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 137:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:574
		{
			if yyDollar[5].nodesWithEndToken.endToken.Value == ";" {
				yyVAL.node = stmt.NewAltSwitch(yyDollar[3].node, yyDollar[5].nodesWithEndToken.nodes)
			} else {
				yyVAL.node = stmt.NewSwitch(yyDollar[3].node, yyDollar[5].nodesWithEndToken.nodes)
			}
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].nodesWithEndToken.endToken))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 138:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:584
		{
			yyVAL.node = stmt.NewBreak(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 139:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:590
		{
			yyVAL.node = stmt.NewContinue(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 140:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:596
		{
			yyVAL.node = stmt.NewReturn(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 141:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:602
		{
			yyVAL.node = stmt.NewGlobal(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 142:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:608
		{
			yyVAL.node = stmt.NewStatic(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 143:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:614
		{
			yyVAL.node = stmt.NewEcho(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 144:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:620
		{
			yyVAL.node = stmt.NewInlineHtml(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 145:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:626
		{
			yyVAL.node = stmt.NewExpression(yyDollar[1].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 146:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:632
		{
			yyVAL.node = stmt.NewUnset(yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 147:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:638
		{
			if yyDollar[7].altSyntaxNode.isAlt {
				yyVAL.node = stmt.NewAltForeach(yyDollar[3].node, nil, yyDollar[5].foreachVariable.node, yyDollar[7].altSyntaxNode.node, yyDollar[5].foreachVariable.byRef)
			} else {
				yyVAL.node = stmt.NewForeach(yyDollar[3].node, nil, yyDollar[5].foreachVariable.node, yyDollar[7].altSyntaxNode.node, yyDollar[5].foreachVariable.byRef)
			}
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[7].altSyntaxNode.node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 148:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line php7/php7.y:648
		{
			if yyDollar[9].altSyntaxNode.isAlt {
				yyVAL.node = stmt.NewAltForeach(yyDollar[3].node, yyDollar[5].node, yyDollar[7].foreachVariable.node, yyDollar[9].altSyntaxNode.node, yyDollar[7].foreachVariable.byRef)
			} else {
				yyVAL.node = stmt.NewForeach(yyDollar[3].node, yyDollar[5].node, yyDollar[7].foreachVariable.node, yyDollar[9].altSyntaxNode.node, yyDollar[7].foreachVariable.byRef)
			}
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[9].altSyntaxNode.node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 149:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:658
		{
			yyVAL.node = stmt.NewDeclare(yyDollar[3].list, yyDollar[5].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 150:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:664
		{
			yyVAL.node = stmt.NewNop()
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 151:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:670
		{
			if yyDollar[6].node == nil {
				yyVAL.node = stmt.NewTry(yyDollar[3].list, yyDollar[5].list, yyDollar[6].node)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[5].list))
			} else {
				yyVAL.node = stmt.NewTry(yyDollar[3].list, yyDollar[5].list, yyDollar[6].node)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[6].node))
			}

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 152:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:682
		{
			yyVAL.node = stmt.NewThrow(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 153:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:688
		{
			label := node.NewIdentifier(yyDollar[2].token.Value)
			yylex.(*Parser).positions.AddPosition(label, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node = stmt.NewGoto(label)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(label, yyDollar[2].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 154:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:698
		{
			label := node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(label, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = stmt.NewLabel(label)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			yylex.(*Parser).comments.AddComments(label, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 155:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:709
		{
			yyVAL.list = []node.Node{}
		}
	case 156:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line php7/php7.y:711
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[5].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[5].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[5].token))
			catch := stmt.NewCatch(yyDollar[4].list, variable, yyDollar[8].list)
			yylex.(*Parser).positions.AddPosition(catch, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[2].token, yyDollar[9].token))
			yyVAL.list = append(yyDollar[1].list, catch)

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[5].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[5].token.Comments())
			yylex.(*Parser).comments.AddComments(catch, yyDollar[2].token.Comments())
		}
	case 157:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:726
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 158:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:727
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 159:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:731
		{
			yyVAL.node = nil
		}
	case 160:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:733
		{
			yyVAL.node = stmt.NewFinally(yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 161:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:741
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 162:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:742
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 163:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:746
		{
			yyVAL.node = yyDollar[1].node
		}
	case 164:
		yyDollar = yyS[yypt-11 : yypt+1]
		//line php7/php7.y:751
		{
			name := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = stmt.NewFunction(name, yyDollar[2].boolWithToken.value, yyDollar[6].list, yyDollar[8].node, yyDollar[10].list, yyDollar[4].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[11].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 165:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:763
		{
			yyVAL.boolWithToken = boolWithToken{false, nil}
		}
	case 166:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:764
		{
			yyVAL.boolWithToken = boolWithToken{true, &yyDollar[1].token}
		}
	case 167:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:768
		{
			yyVAL.boolWithToken = boolWithToken{false, nil}
		}
	case 168:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:769
		{
			yyVAL.boolWithToken = boolWithToken{true, &yyDollar[1].token}
		}
	case 169:
		yyDollar = yyS[yypt-9 : yypt+1]
		//line php7/php7.y:774
		{
			name := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = stmt.NewClass(name, yyDollar[1].list, nil, yyDollar[4].node, yyDollar[5].list, yyDollar[8].list, yyDollar[6].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewOptionalListTokensPosition(yyDollar[1].list, yyDollar[2].token, yyDollar[9].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 170:
		yyDollar = yyS[yypt-8 : yypt+1]
		//line php7/php7.y:784
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node = stmt.NewClass(name, nil, nil, yyDollar[3].node, yyDollar[4].list, yyDollar[7].list, yyDollar[5].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[8].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[2].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 171:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:796
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 172:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:797
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 173:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:802
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 174:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:808
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 175:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:817
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node = stmt.NewTrait(name, yyDollar[5].list, yyDollar[3].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[2].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 176:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:830
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node = stmt.NewInterface(name, yyDollar[3].list, yyDollar[6].list, yyDollar[4].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[7].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[2].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 177:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:842
		{
			yyVAL.node = nil
		}
	case 178:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:843
		{
			yyVAL.node = yyDollar[2].node
		}
	case 179:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:847
		{
			yyVAL.list = nil
		}
	case 180:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:848
		{
			yyVAL.list = yyDollar[2].list
		}
	case 181:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:852
		{
			yyVAL.list = nil
		}
	case 182:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:853
		{
			yyVAL.list = yyDollar[2].list
		}
	case 183:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:857
		{
			yyVAL.foreachVariable = foreachVariable{yyDollar[1].node, false}
		}
	case 184:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:858
		{
			yyVAL.foreachVariable = foreachVariable{yyDollar[2].node, true}
		}
	case 185:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:860
		{
			list := expr.NewList(yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(list, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yyVAL.foreachVariable = foreachVariable{list, false}
			yylex.(*Parser).comments.AddComments(list, yyDollar[1].token.Comments())
		}
	case 186:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:867
		{
			list := expr.NewShortList(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(list, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yyVAL.foreachVariable = foreachVariable{list, false}
			yylex.(*Parser).comments.AddComments(list, yyDollar[1].token.Comments())
		}
	case 187:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:877
		{
			yyVAL.altSyntaxNode = altSyntaxNode{yyDollar[1].node, false}
		}
	case 188:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:879
		{
			yyVAL.altSyntaxNode = altSyntaxNode{stmt.NewStmtList(yyDollar[2].list), true}
			yylex.(*Parser).positions.AddPosition(yyVAL.altSyntaxNode.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
		}
	case 189:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:887
		{
			yyVAL.altSyntaxNode = altSyntaxNode{yyDollar[1].node, false}
		}
	case 190:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:889
		{
			yyVAL.altSyntaxNode = altSyntaxNode{stmt.NewStmtList(yyDollar[2].list), true}
			yylex.(*Parser).positions.AddPosition(yyVAL.altSyntaxNode.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
		}
	case 191:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:896
		{
			yyVAL.node = yyDollar[1].node
		}
	case 192:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:898
		{
			yyVAL.node = stmt.NewStmtList(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 193:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:906
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{yyDollar[2].list, yyDollar[3].token}
		}
	case 194:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:907
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{yyDollar[3].list, yyDollar[4].token}
		}
	case 195:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:908
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{yyDollar[2].list, yyDollar[4].token}
		}
	case 196:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:909
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{yyDollar[3].list, yyDollar[5].token}
		}
	case 197:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:913
		{
			yyVAL.list = []node.Node{}
		}
	case 198:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:915
		{
			_case := stmt.NewCase(yyDollar[3].node, yyDollar[5].list)
			yylex.(*Parser).positions.AddPosition(_case, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[5].list))
			yyVAL.list = append(yyDollar[1].list, _case)
			yylex.(*Parser).comments.AddComments(_case, yyDollar[2].token.Comments())
		}
	case 199:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:922
		{
			_default := stmt.NewDefault(yyDollar[4].list)
			yylex.(*Parser).positions.AddPosition(_default, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[4].list))
			yyVAL.list = append(yyDollar[1].list, _default)
			yylex.(*Parser).comments.AddComments(_default, yyDollar[2].token.Comments())
		}
	case 202:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:937
		{
			yyVAL.altSyntaxNode = altSyntaxNode{yyDollar[1].node, false}
		}
	case 203:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:939
		{
			yyVAL.altSyntaxNode = altSyntaxNode{stmt.NewStmtList(yyDollar[2].list), true}
			yylex.(*Parser).positions.AddPosition(yyVAL.altSyntaxNode.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
		}
	case 204:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:947
		{
			yyVAL.node = stmt.NewIf(yyDollar[3].node, yyDollar[5].node, nil, nil)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 205:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:953
		{
			_elseIf := stmt.NewElseIf(yyDollar[4].node, yyDollar[6].node)
			yylex.(*Parser).positions.AddPosition(_elseIf, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[6].node))
			yyVAL.node = yyDollar[1].node.(*stmt.If).AddElseIf(_elseIf)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[6].node))

			yylex.(*Parser).comments.AddComments(_elseIf, yyDollar[2].token.Comments())
		}
	case 206:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:964
		{
			yyVAL.node = yyDollar[1].node
		}
	case 207:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:966
		{
			_else := stmt.NewElse(yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(_else, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[2].token, yyDollar[3].node))
			yyVAL.node = yyDollar[1].node.(*stmt.If).SetElse(_else)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[2].token.Comments())
		}
	case 208:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:978
		{
			stmts := stmt.NewStmtList(yyDollar[6].list)
			yylex.(*Parser).positions.AddPosition(stmts, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[6].list))
			yyVAL.node = stmt.NewAltIf(yyDollar[3].node, stmts, nil, nil)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[1].token, yyDollar[6].list))

			yylex.(*Parser).comments.AddComments(stmts, yyDollar[5].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 209:
		yyDollar = yyS[yypt-7 : yypt+1]
		//line php7/php7.y:988
		{
			stmts := stmt.NewStmtList(yyDollar[7].list)
			yylex.(*Parser).positions.AddPosition(stmts, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[7].list))
			_elseIf := stmt.NewAltElseIf(yyDollar[4].node, stmts)
			yylex.(*Parser).positions.AddPosition(_elseIf, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[7].list))
			yyVAL.node = yyDollar[1].node.(*stmt.AltIf).AddElseIf(_elseIf)

			yylex.(*Parser).comments.AddComments(stmts, yyDollar[6].token.Comments())
			yylex.(*Parser).comments.AddComments(_elseIf, yyDollar[2].token.Comments())
		}
	case 210:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1002
		{
			yyVAL.node = yyDollar[1].node
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))
		}
	case 211:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1007
		{
			stmts := stmt.NewStmtList(yyDollar[4].list)
			yylex.(*Parser).positions.AddPosition(stmts, yylex.(*Parser).positionBuilder.NewNodeListPosition(yyDollar[4].list))
			_else := stmt.NewAltElse(stmts)
			yylex.(*Parser).positions.AddPosition(_else, yylex.(*Parser).positionBuilder.NewTokenNodeListPosition(yyDollar[2].token, yyDollar[4].list))
			yyVAL.node = yyDollar[1].node.(*stmt.AltIf).SetElse(_else)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[6].token))

			yylex.(*Parser).comments.AddComments(stmts, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(_else, yyDollar[2].token.Comments())
		}
	case 212:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1021
		{
			yyVAL.list = yyDollar[1].list
		}
	case 213:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1022
		{
			yyVAL.list = nil
		}
	case 214:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1026
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 215:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1027
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 216:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1032
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[4].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[4].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[4].token.Comments())

			if yyDollar[1].node != nil {
				yyVAL.node = node.NewParameter(yyDollar[1].node, variable, nil, yyDollar[2].boolWithToken.value, yyDollar[3].boolWithToken.value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))
				yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
			} else if yyDollar[2].boolWithToken.value == true {
				yyVAL.node = node.NewParameter(yyDollar[1].node, variable, nil, yyDollar[2].boolWithToken.value, yyDollar[3].boolWithToken.value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(*yyDollar[2].boolWithToken.token, yyDollar[4].token))
				yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[2].boolWithToken.token.Comments())
			} else if yyDollar[3].boolWithToken.value == true {
				yyVAL.node = node.NewParameter(yyDollar[1].node, variable, nil, yyDollar[2].boolWithToken.value, yyDollar[3].boolWithToken.value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(*yyDollar[3].boolWithToken.token, yyDollar[4].token))
				yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[3].boolWithToken.token.Comments())
			} else {
				yyVAL.node = node.NewParameter(yyDollar[1].node, variable, nil, yyDollar[2].boolWithToken.value, yyDollar[3].boolWithToken.value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
				yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[4].token.Comments())
			}
		}
	case 217:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1060
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[4].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[4].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[4].token.Comments())

			if yyDollar[1].node != nil {
				yyVAL.node = node.NewParameter(yyDollar[1].node, variable, yyDollar[6].node, yyDollar[2].boolWithToken.value, yyDollar[3].boolWithToken.value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[6].node))
				yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
			} else if yyDollar[2].boolWithToken.value == true {
				yyVAL.node = node.NewParameter(yyDollar[1].node, variable, yyDollar[6].node, yyDollar[2].boolWithToken.value, yyDollar[3].boolWithToken.value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(*yyDollar[2].boolWithToken.token, yyDollar[6].node))
				yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[2].boolWithToken.token.Comments())
			} else if yyDollar[3].boolWithToken.value == true {
				yyVAL.node = node.NewParameter(yyDollar[1].node, variable, yyDollar[6].node, yyDollar[2].boolWithToken.value, yyDollar[3].boolWithToken.value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(*yyDollar[3].boolWithToken.token, yyDollar[6].node))
				yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[3].boolWithToken.token.Comments())
			} else {
				yyVAL.node = node.NewParameter(yyDollar[1].node, variable, yyDollar[6].node, yyDollar[2].boolWithToken.value, yyDollar[3].boolWithToken.value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[4].token, yyDollar[6].node))
				yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[4].token.Comments())
			}
		}
	case 218:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1090
		{
			yyVAL.node = nil
		}
	case 219:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1091
		{
			yyVAL.node = yyDollar[1].node
		}
	case 220:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1095
		{
			yyVAL.node = yyDollar[1].node
		}
	case 221:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1097
		{
			yyVAL.node = node.NewNullable(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 222:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1106
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 223:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1112
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 224:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1117
		{
			yyVAL.node = yyDollar[1].node
		}
	case 225:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1121
		{
			yyVAL.node = nil
		}
	case 226:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1122
		{
			yyVAL.node = yyDollar[2].node
		}
	case 227:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1126
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{[]node.Node{}, yyDollar[2].token}
		}
	case 228:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1127
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{yyDollar[2].list, yyDollar[4].token}
		}
	case 229:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1131
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 230:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1132
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 231:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1137
		{
			yyVAL.node = node.NewArgument(yyDollar[1].node, false, false)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 232:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1143
		{
			yyVAL.node = node.NewArgument(yyDollar[2].node, true, false)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 233:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1151
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 234:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1152
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 235:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1156
		{
			yyVAL.node = yyDollar[1].node
		}
	case 236:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1160
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 237:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1161
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 238:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1166
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = stmt.NewStaticVar(variable, nil)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 239:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1179
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = stmt.NewStaticVar(variable, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 240:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1194
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 241:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1195
		{
			yyVAL.list = []node.Node{}
		}
	case 242:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1200
		{
			yyVAL.node = stmt.NewPropertyList(yyDollar[1].list, yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeListTokenPosition(yyDollar[1].list, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 243:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1206
		{
			yyVAL.node = stmt.NewClassConstList(yyDollar[1].list, yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewOptionalListTokensPosition(yyDollar[1].list, yyDollar[2].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 244:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1212
		{
			yyVAL.node = stmt.NewTraitUse(yyDollar[2].list, yyDollar[3].nodesWithEndToken.nodes)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].nodesWithEndToken.endToken))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 245:
		yyDollar = yyS[yypt-10 : yypt+1]
		//line php7/php7.y:1218
		{
			name := node.NewIdentifier(yyDollar[4].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			yyVAL.node = stmt.NewClassMethod(name, yyDollar[1].list, yyDollar[3].boolWithToken.value, yyDollar[7].list, yyDollar[9].node, yyDollar[10].nodesWithEndToken.nodes, yyDollar[5].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewOptionalListTokensPosition(yyDollar[1].list, yyDollar[2].token, yyDollar[10].nodesWithEndToken.endToken))

			yylex.(*Parser).comments.AddComments(name, yyDollar[4].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).listGetFirstNodeComments(yyDollar[1].list))
		}
	case 246:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1230
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 247:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1231
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 248:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1235
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{nil, yyDollar[1].token}
		}
	case 249:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1236
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{nil, yyDollar[2].token}
		}
	case 250:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1237
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{yyDollar[2].list, yyDollar[3].token}
		}
	case 251:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1241
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 252:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1242
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 253:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1246
		{
			yyVAL.node = yyDollar[1].node
		}
	case 254:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1247
		{
			yyVAL.node = yyDollar[1].node
		}
	case 255:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1252
		{
			yyVAL.node = stmt.NewTraitUsePrecedence(yyDollar[1].node, yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeNodeListPosition(yyDollar[1].node, yyDollar[3].list))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 256:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1261
		{
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(alias, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, nil, alias)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(alias, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 257:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1271
		{
			alias := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(alias, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, nil, alias)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(alias, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 258:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1281
		{
			alias := node.NewIdentifier(yyDollar[4].token.Value)
			yylex.(*Parser).positions.AddPosition(alias, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[4].token))
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, yyDollar[3].node, alias)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))

			yylex.(*Parser).comments.AddComments(alias, yyDollar[4].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 259:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1291
		{
			yyVAL.node = stmt.NewTraitUseAlias(yyDollar[1].node, yyDollar[3].node, nil)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 260:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1300
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = stmt.NewTraitMethodRef(nil, name)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 261:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1309
		{
			yyVAL.node = yyDollar[1].node
		}
	case 262:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1314
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(target, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = stmt.NewTraitMethodRef(yyDollar[1].node, target)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(target, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 263:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1326
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{nil, yyDollar[1].token}
		}
	case 264:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1327
		{
			yyVAL.nodesWithEndToken = &nodesWithEndToken{yyDollar[2].list, yyDollar[3].token}
		}
	case 265:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1331
		{
			yyVAL.list = yyDollar[1].list
		}
	case 266:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1333
		{
			modifier := node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(modifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.list = []node.Node{modifier}
			yylex.(*Parser).comments.AddComments(modifier, yyDollar[1].token.Comments())
		}
	case 267:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1342
		{
			yyVAL.list = nil
		}
	case 268:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1343
		{
			yyVAL.list = yyDollar[1].list
		}
	case 269:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1347
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 270:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1348
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 271:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1353
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 272:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1359
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 273:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1365
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 274:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1371
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 275:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1377
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 276:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1383
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 277:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1391
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 278:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1392
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 279:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1397
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = stmt.NewProperty(variable, nil, yyDollar[2].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 280:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1410
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = stmt.NewProperty(variable, yyDollar[3].node, yyDollar[4].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 281:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1425
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 282:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1426
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 283:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1431
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = stmt.NewConstant(name, yyDollar[3].node, yyDollar[4].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			yylex.(*Parser).comments.AddComments(name, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 284:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1444
		{
			name := node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = stmt.NewConstant(name, yyDollar[3].node, yyDollar[4].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[3].node))

			yylex.(*Parser).comments.AddComments(name, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 285:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1456
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 286:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1457
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 287:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1461
		{
			yyVAL.node = yyDollar[1].node
		}
	case 288:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1465
		{
			yyVAL.list = nil
		}
	case 289:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1466
		{
			yyVAL.list = yyDollar[1].list
		}
	case 290:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1469
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 291:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1470
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 292:
		yyDollar = yyS[yypt-8 : yypt+1]
		//line php7/php7.y:1475
		{
			if yyDollar[2].nodesWithEndToken != nil {
				yyVAL.node = stmt.NewClass(nil, nil, yyDollar[2].nodesWithEndToken.nodes, yyDollar[3].node, yyDollar[4].list, yyDollar[7].list, yyDollar[5].str)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[8].token))
			} else {
				yyVAL.node = stmt.NewClass(nil, nil, nil, yyDollar[3].node, yyDollar[4].list, yyDollar[7].list, yyDollar[5].str)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[8].token))
			}

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 293:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1490
		{
			if yyDollar[3].nodesWithEndToken != nil {
				yyVAL.node = expr.NewNew(yyDollar[2].node, yyDollar[3].nodesWithEndToken.nodes)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].nodesWithEndToken.endToken))
			} else {
				yyVAL.node = expr.NewNew(yyDollar[2].node, nil)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			}

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 294:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1501
		{
			yyVAL.node = expr.NewNew(yyDollar[2].node, nil)
		}
	case 295:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:1506
		{
			list := expr.NewList(yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(list, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yyVAL.node = assign.NewAssign(list, yyDollar[6].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[6].node))

			yylex.(*Parser).comments.AddComments(list, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 296:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:1516
		{
			shortList := expr.NewShortList(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(shortList, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yyVAL.node = assign.NewAssign(shortList, yyDollar[5].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[5].node))

			yylex.(*Parser).comments.AddComments(shortList, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 297:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1526
		{
			yyVAL.node = assign.NewAssign(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 298:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1532
		{
			yyVAL.node = assign.NewReference(yyDollar[1].node, yyDollar[4].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 299:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1538
		{
			yyVAL.node = expr.NewClone(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 300:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1544
		{
			yyVAL.node = assign.NewPlus(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 301:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1550
		{
			yyVAL.node = assign.NewMinus(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 302:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1556
		{
			yyVAL.node = assign.NewMul(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 303:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1562
		{
			yyVAL.node = assign.NewPow(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 304:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1568
		{
			yyVAL.node = assign.NewDiv(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 305:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1574
		{
			yyVAL.node = assign.NewConcat(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 306:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1580
		{
			yyVAL.node = assign.NewMod(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 307:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1586
		{
			yyVAL.node = assign.NewBitwiseAnd(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 308:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1592
		{
			yyVAL.node = assign.NewBitwiseOr(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 309:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1598
		{
			yyVAL.node = assign.NewBitwiseXor(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 310:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1604
		{
			yyVAL.node = assign.NewShiftLeft(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 311:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1610
		{
			yyVAL.node = assign.NewShiftRight(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 312:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1616
		{
			yyVAL.node = expr.NewPostInc(yyDollar[1].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 313:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1622
		{
			yyVAL.node = expr.NewPreInc(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 314:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1628
		{
			yyVAL.node = expr.NewPostDec(yyDollar[1].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 315:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1634
		{
			yyVAL.node = expr.NewPreDec(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 316:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1640
		{
			yyVAL.node = binary.NewBooleanOr(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 317:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1646
		{
			yyVAL.node = binary.NewBooleanAnd(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 318:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1652
		{
			yyVAL.node = binary.NewLogicalOr(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 319:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1658
		{
			yyVAL.node = binary.NewLogicalAnd(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 320:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1664
		{
			yyVAL.node = binary.NewLogicalXor(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 321:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1670
		{
			yyVAL.node = binary.NewBitwiseOr(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 322:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1676
		{
			yyVAL.node = binary.NewBitwiseAnd(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 323:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1682
		{
			yyVAL.node = binary.NewBitwiseXor(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 324:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1688
		{
			yyVAL.node = binary.NewConcat(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 325:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1694
		{
			yyVAL.node = binary.NewPlus(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 326:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1700
		{
			yyVAL.node = binary.NewMinus(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 327:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1706
		{
			yyVAL.node = binary.NewMul(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 328:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1712
		{
			yyVAL.node = binary.NewPow(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 329:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1718
		{
			yyVAL.node = binary.NewDiv(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 330:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1724
		{
			yyVAL.node = binary.NewMod(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 331:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1730
		{
			yyVAL.node = binary.NewShiftLeft(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 332:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1736
		{
			yyVAL.node = binary.NewShiftRight(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 333:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1742
		{
			yyVAL.node = expr.NewUnaryPlus(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 334:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1748
		{
			yyVAL.node = expr.NewUnaryMinus(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 335:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1754
		{
			yyVAL.node = expr.NewBooleanNot(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 336:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1760
		{
			yyVAL.node = expr.NewBitwiseNot(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 337:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1766
		{
			yyVAL.node = binary.NewIdentical(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 338:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1772
		{
			yyVAL.node = binary.NewNotIdentical(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 339:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1778
		{
			yyVAL.node = binary.NewEqual(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 340:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1784
		{
			yyVAL.node = binary.NewNotEqual(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 341:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1790
		{
			yyVAL.node = binary.NewSmaller(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 342:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1796
		{
			yyVAL.node = binary.NewSmallerOrEqual(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 343:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1802
		{
			yyVAL.node = binary.NewGreater(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 344:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1808
		{
			yyVAL.node = binary.NewGreaterOrEqual(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 345:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1814
		{
			yyVAL.node = binary.NewSpaceship(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 346:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1820
		{
			yyVAL.node = expr.NewInstanceOf(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 347:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1825
		{
			yyVAL.node = yyDollar[2].node
		}
	case 348:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1826
		{
			yyVAL.node = yyDollar[1].node
		}
	case 349:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:1828
		{
			yyVAL.node = expr.NewTernary(yyDollar[1].node, yyDollar[3].node, yyDollar[5].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[5].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 350:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1834
		{
			yyVAL.node = expr.NewTernary(yyDollar[1].node, nil, yyDollar[4].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 351:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1840
		{
			yyVAL.node = binary.NewCoalesce(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 352:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1845
		{
			yyVAL.node = yyDollar[1].node
		}
	case 353:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1847
		{
			yyVAL.node = cast.NewInt(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 354:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1853
		{
			yyVAL.node = cast.NewDouble(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 355:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1859
		{
			yyVAL.node = cast.NewString(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 356:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1865
		{
			yyVAL.node = cast.NewArray(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 357:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1871
		{
			yyVAL.node = cast.NewObject(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 358:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1877
		{
			yyVAL.node = cast.NewBool(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 359:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1883
		{
			yyVAL.node = cast.NewUnset(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 360:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1889
		{
			if strings.EqualFold(yyDollar[1].token.Value, "die") {
				yyVAL.node = expr.NewDie(yyDollar[2].node)
			} else {
				yyVAL.node = expr.NewExit(yyDollar[2].node)
			}
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 361:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1899
		{
			yyVAL.node = expr.NewErrorSuppress(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 362:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1904
		{
			yyVAL.node = yyDollar[1].node
		}
	case 363:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1906
		{
			yyVAL.node = expr.NewShellExec(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 364:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1912
		{
			yyVAL.node = expr.NewPrint(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 365:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1918
		{
			yyVAL.node = expr.NewYield(nil, nil)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 366:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1924
		{
			yyVAL.node = expr.NewYield(nil, yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 367:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1930
		{
			yyVAL.node = expr.NewYield(yyDollar[2].node, yyDollar[4].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[4].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 368:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1936
		{
			yyVAL.node = expr.NewYieldFrom(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 369:
		yyDollar = yyS[yypt-11 : yypt+1]
		//line php7/php7.y:1942
		{
			yyVAL.node = expr.NewClosure(yyDollar[5].list, yyDollar[7].list, yyDollar[8].node, yyDollar[10].list, false, yyDollar[2].boolWithToken.value, yyDollar[3].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[11].token))

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 370:
		yyDollar = yyS[yypt-12 : yypt+1]
		//line php7/php7.y:1949
		{
			yyVAL.node = expr.NewClosure(yyDollar[6].list, yyDollar[8].list, yyDollar[9].node, yyDollar[11].list, true, yyDollar[3].boolWithToken.value, yyDollar[4].str)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[12].token))

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 371:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1958
		{
			yyVAL.str = yylex.(*Parser).PhpDocComment
			yylex.(*Parser).PhpDocComment = ""
		}
	case 372:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1962
		{
			yyVAL.boolWithToken = boolWithToken{false, nil}
		}
	case 373:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1963
		{
			yyVAL.boolWithToken = boolWithToken{true, &yyDollar[1].token}
		}
	case 374:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:1967
		{
			yyVAL.list = []node.Node{}
		}
	case 375:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:1968
		{
			yyVAL.list = yyDollar[3].list
		}
	case 376:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:1972
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 377:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1973
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 378:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:1978
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = expr.NewClosureUse(variable, false)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 379:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:1991
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[2].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node = expr.NewClosureUse(variable, true)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[2].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 380:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2007
		{
			yyVAL.node = expr.NewFunctionCall(yyDollar[1].node, yyDollar[2].nodesWithEndToken.nodes)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].nodesWithEndToken.endToken))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 381:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2013
		{
			yyVAL.node = expr.NewStaticCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].nodesWithEndToken.nodes)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].nodesWithEndToken.endToken))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 382:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2019
		{
			yyVAL.node = expr.NewStaticCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].nodesWithEndToken.nodes)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].nodesWithEndToken.endToken))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 383:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2025
		{
			yyVAL.node = expr.NewFunctionCall(yyDollar[1].node, yyDollar[2].nodesWithEndToken.nodes)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[2].nodesWithEndToken.endToken))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 384:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2034
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 385:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2039
		{
			yyVAL.node = yyDollar[1].node
		}
	case 386:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2043
		{
			yyVAL.node = yyDollar[1].node
		}
	case 387:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2044
		{
			yyVAL.node = yyDollar[1].node
		}
	case 388:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2048
		{
			yyVAL.node = nil
		}
	case 389:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2049
		{
			yyVAL.node = yyDollar[2].node
		}
	case 390:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2053
		{
			yyVAL.list = []node.Node{}
		}
	case 391:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2054
		{
			yyVAL.list = []node.Node{scalar.NewEncapsedStringPart(yyDollar[1].token.Value)}
		}
	case 392:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2055
		{
			yyVAL.list = yyDollar[1].list
		}
	case 393:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2059
		{
			yyVAL.nodesWithEndToken = nil
		}
	case 394:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2060
		{
			yyVAL.nodesWithEndToken = yyDollar[1].nodesWithEndToken
		}
	case 395:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2065
		{
			yyVAL.node = expr.NewArray(yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 396:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2071
		{
			yyVAL.node = expr.NewShortArray(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 397:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2077
		{
			yyVAL.node = scalar.NewString(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 398:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2086
		{
			yyVAL.node = scalar.NewLnumber(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 399:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2092
		{
			yyVAL.node = scalar.NewDnumber(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 400:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2098
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 401:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2104
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 402:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2110
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 403:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2116
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 404:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2122
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 405:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2128
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 406:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2134
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 407:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2140
		{
			yyVAL.node = scalar.NewMagicConstant(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 408:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2146
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[2].token.Value)
			yylex.(*Parser).positions.AddPosition(encapsed, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yylex.(*Parser).comments.AddComments(encapsed, yyDollar[2].token.Comments())

			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, []node.Node{encapsed})
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 409:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2156
		{
			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, nil)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 410:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2162
		{
			yyVAL.node = scalar.NewEncapsed(yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 411:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2168
		{
			yyVAL.node = scalar.NewHeredoc(yyDollar[1].token.Value, yyDollar[2].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 412:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2173
		{
			yyVAL.node = yyDollar[1].node
		}
	case 413:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2174
		{
			yyVAL.node = yyDollar[1].node
		}
	case 414:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2179
		{
			yyVAL.node = expr.NewConstFetch(yyDollar[1].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 415:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2185
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(target, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = expr.NewClassConstFetch(yyDollar[1].node, target)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(target, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 416:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2195
		{
			target := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(target, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = expr.NewClassConstFetch(yyDollar[1].node, target)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(target, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 417:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2207
		{
			yyVAL.node = yyDollar[1].node
		}
	case 418:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2208
		{
			yyVAL.node = yyDollar[1].node
		}
	case 419:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2212
		{
			yyVAL.node = nil
		}
	case 420:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2213
		{
			yyVAL.node = yyDollar[1].node
		}
	case 421:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2217
		{
			yyVAL.node = yyDollar[1].node
		}
	case 422:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2221
		{
			yyVAL.node = yyDollar[1].node
		}
	case 423:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2222
		{
			yyVAL.node = yyDollar[2].node
		}
	case 424:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2223
		{
			yyVAL.node = yyDollar[1].node
		}
	case 425:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2227
		{
			yyVAL.node = yyDollar[1].node
		}
	case 426:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2228
		{
			yyVAL.node = yyDollar[2].node
		}
	case 427:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2229
		{
			yyVAL.node = yyDollar[1].node
		}
	case 428:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2233
		{
			yyVAL.node = yyDollar[1].node
		}
	case 429:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2235
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 430:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2241
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 431:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2247
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 432:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2253
		{
			yyVAL.node = expr.NewMethodCall(yyDollar[1].node, yyDollar[3].node, yyDollar[4].nodesWithEndToken.nodes)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].nodesWithEndToken.endToken))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 433:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2258
		{
			yyVAL.node = yyDollar[1].node
		}
	case 434:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2262
		{
			yyVAL.node = yyDollar[1].node
		}
	case 435:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2263
		{
			yyVAL.node = yyDollar[1].node
		}
	case 436:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2265
		{
			yyVAL.node = expr.NewPropertyFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 437:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2274
		{
			name := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = expr.NewVariable(name)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 438:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2284
		{
			yyVAL.node = expr.NewVariable(yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 439:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2290
		{
			yyVAL.node = expr.NewVariable(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 440:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2299
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 441:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2305
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 442:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2313
		{
			yyVAL.node = yyDollar[1].node
		}
	case 443:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2315
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 444:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2321
		{
			yyVAL.node = expr.NewArrayDimFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 445:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2327
		{
			yyVAL.node = expr.NewPropertyFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 446:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2333
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 447:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2339
		{
			yyVAL.node = expr.NewStaticPropertyFetch(yyDollar[1].node, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 448:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2348
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 449:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2353
		{
			yyVAL.node = yyDollar[2].node
		}
	case 450:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2354
		{
			yyVAL.node = yyDollar[1].node
		}
	case 451:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2359
		{
			yyVAL.node = node.NewIdentifier(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 452:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2364
		{
			yyVAL.node = yyDollar[2].node
		}
	case 453:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2365
		{
			yyVAL.node = yyDollar[1].node
		}
	case 454:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2370
		{
			if yyDollar[1].list[len(yyDollar[1].list)-1] == nil {
				yyVAL.list = yyDollar[1].list[:len(yyDollar[1].list)-1]
			} else {
				yyVAL.list = yyDollar[1].list
			}
		}
	case 455:
		yyDollar = yyS[yypt-0 : yypt+1]
		//line php7/php7.y:2380
		{
			yyVAL.node = nil
		}
	case 456:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2381
		{
			yyVAL.node = yyDollar[1].node
		}
	case 457:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2386
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 458:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2387
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 459:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2392
		{
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, yyDollar[3].node, false)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[3].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 460:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2398
		{
			yyVAL.node = expr.NewArrayItem(nil, yyDollar[1].node, false)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodePosition(yyDollar[1].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 461:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2404
		{
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, yyDollar[4].node, true)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodesPosition(yyDollar[1].node, yyDollar[4].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 462:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2410
		{
			yyVAL.node = expr.NewArrayItem(nil, yyDollar[2].node, true)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 463:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:2416
		{
			// TODO: Cannot use list() as standalone expression
			list := expr.NewList(yyDollar[5].list)
			yylex.(*Parser).positions.AddPosition(list, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[3].token, yyDollar[6].token))
			yyVAL.node = expr.NewArrayItem(yyDollar[1].node, list, false)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewNodeTokenPosition(yyDollar[1].node, yyDollar[6].token))

			yylex.(*Parser).comments.AddComments(list, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yylex.(*Parser).comments[yyDollar[1].node])
		}
	case 464:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2427
		{
			// TODO: Cannot use list() as standalone expression
			list := expr.NewList(yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(list, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yyVAL.node = expr.NewArrayItem(nil, list, false)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			yylex.(*Parser).comments.AddComments(list, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 465:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2440
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[2].node)
		}
	case 466:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2442
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[2].token.Value)
			yylex.(*Parser).positions.AddPosition(encapsed, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.list = append(yyDollar[1].list, encapsed)
			yylex.(*Parser).comments.AddComments(encapsed, yyDollar[2].token.Comments())
		}
	case 467:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2448
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 468:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2450
		{
			encapsed := scalar.NewEncapsedStringPart(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(encapsed, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.list = []node.Node{encapsed, yyDollar[2].node}
			yylex.(*Parser).comments.AddComments(encapsed, yyDollar[1].token.Comments())
		}
	case 469:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2460
		{
			name := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = expr.NewVariable(name)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 470:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2470
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = expr.NewArrayDimFetch(variable, yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 471:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2483
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			fetch := node.NewIdentifier(yyDollar[3].token.Value)
			yylex.(*Parser).positions.AddPosition(fetch, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[3].token))
			yyVAL.node = expr.NewPropertyFetch(variable, fetch)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(fetch, yyDollar[3].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 472:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2499
		{
			yyVAL.node = expr.NewVariable(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 473:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2505
		{
			name := node.NewIdentifier(yyDollar[2].token.Value)
			yylex.(*Parser).positions.AddPosition(name, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node = expr.NewVariable(name)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[3].token))

			yylex.(*Parser).comments.AddComments(name, yyDollar[2].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 474:
		yyDollar = yyS[yypt-6 : yypt+1]
		//line php7/php7.y:2515
		{
			identifier := node.NewIdentifier(yyDollar[2].token.Value)
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			variable := expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(variable, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[2].token))
			yyVAL.node = expr.NewArrayDimFetch(variable, yyDollar[4].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[6].token))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[2].token.Comments())
			yylex.(*Parser).comments.AddComments(variable, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 475:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2528
		{
			yyVAL.node = yyDollar[2].node
		}
	case 476:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2532
		{
			yyVAL.node = scalar.NewString(yyDollar[1].token.Value)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 477:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2538
		{
			// TODO: add option to handle 64 bit integer
			if _, err := strconv.Atoi(yyDollar[1].token.Value); err == nil {
				yyVAL.node = scalar.NewLnumber(yyDollar[1].token.Value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			} else {
				yyVAL.node = scalar.NewString(yyDollar[1].token.Value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			}
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 478:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2550
		{
			// TODO: add option to handle 64 bit integer
			if _, err := strconv.Atoi(yyDollar[2].token.Value); err == nil {
				lnumber := scalar.NewLnumber(yyDollar[2].token.Value)
				yylex.(*Parser).positions.AddPosition(lnumber, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))
				yyVAL.node = expr.NewUnaryMinus(lnumber)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))

				yylex.(*Parser).comments.AddComments(lnumber, yyDollar[1].token.Comments())
			} else {
				yyDollar[2].token.Value = "-" + yyDollar[2].token.Value
				yyVAL.node = scalar.NewString(yyDollar[2].token.Value)
				yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[2].token))
			}

			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 479:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2568
		{
			identifier := node.NewIdentifier(strings.TrimLeft(yyDollar[1].token.Value, "$"))
			yylex.(*Parser).positions.AddPosition(identifier, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))
			yyVAL.node = expr.NewVariable(identifier)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenPosition(yyDollar[1].token))

			yylex.(*Parser).comments.AddComments(identifier, yyDollar[1].token.Comments())
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 480:
		yyDollar = yyS[yypt-5 : yypt+1]
		//line php7/php7.y:2581
		{
			yyVAL.node = expr.NewIsset(yyDollar[3].list)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[5].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 481:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2587
		{
			yyVAL.node = expr.NewEmpty(yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 482:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2593
		{
			yyVAL.node = expr.NewInclude(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 483:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2599
		{
			yyVAL.node = expr.NewIncludeOnce(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 484:
		yyDollar = yyS[yypt-4 : yypt+1]
		//line php7/php7.y:2605
		{
			yyVAL.node = expr.NewEval(yyDollar[3].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokensPosition(yyDollar[1].token, yyDollar[4].token))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 485:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2611
		{
			yyVAL.node = expr.NewRequire(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 486:
		yyDollar = yyS[yypt-2 : yypt+1]
		//line php7/php7.y:2617
		{
			yyVAL.node = expr.NewRequireOnce(yyDollar[2].node)
			yylex.(*Parser).positions.AddPosition(yyVAL.node, yylex.(*Parser).positionBuilder.NewTokenNodePosition(yyDollar[1].token, yyDollar[2].node))
			yylex.(*Parser).comments.AddComments(yyVAL.node, yyDollar[1].token.Comments())
		}
	case 487:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2625
		{
			yyVAL.list = []node.Node{yyDollar[1].node}
		}
	case 488:
		yyDollar = yyS[yypt-3 : yypt+1]
		//line php7/php7.y:2626
		{
			yyVAL.list = append(yyDollar[1].list, yyDollar[3].node)
		}
	case 489:
		yyDollar = yyS[yypt-1 : yypt+1]
		//line php7/php7.y:2630
		{
			yyVAL.node = yyDollar[1].node
		}
	}
	goto yystack /* stack new state and value */
}
